// ad10led4.c - measure input voltage from pot divider using ADC10
// Potential divider disabled between readings set by watchdog timer
// Watchdog -> activate pot div, delay from CCIFG0 -> conversion
// ADC10 default refs (VSS, VCC), internal clock, interrupt on finish
// Home-made analog demo board for MSP430F20xx:
//   10k on P1.5, 20k variable on P1.6, junction to P1.1 with 10nF cap
//   LED active high on P2.6
// Default DCO, no crystal, ACLK from VLO, power from JTAG
// J H Davies, 2007-08-30; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x2.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

#define	LED		P2OUT_bit.P2OUT_6	// Output bit for LED
#define	POTDIV	P1OUT_bit.P1OUT_5	// Output bit for potential divider
// Cycles for delay = 530us * 12kHz = 6.4 -> 7
#define	CHARGING	7				// Cycles of ACLK = VLO for charging

void main (void)
{
	BCSCTL3 = LFXT1S_2;				// Select ACLK from VLO (no crystal)
// Watchdog as interval timer, clear, ACLK / 32768
	WDTCTL = WDTPW | WDTTMSEL | WDTCNTCL | WDTSSEL;
	IE1_bit.WDTIE = 1;				// WDT interval timer interrupts
	P2SEL = 0;						// Digital i/o rather than crystal
	P2OUT = 0;
	P2DIR = BIT6|BIT7;				// Unused outputs driven low
	P1OUT = 0;						// All outputs low
	P1DIR = BIT3|BIT5|BIT6|BIT7;	// Unused outputs driven low
// VCC and VSS refs, sample for 4 cycs, int ref off, ADC off, interrupts
	ADC10CTL0 = SREF_0 | ADC10SHT_0 | ADC10IE;
// Input channel 1, trigger using ADC10SC bit, 2s complement,
//   no clock division, internal osc, single channel single conversions
	ADC10CTL1 = INCH_1|SHS_0|ADC10DF|ADC10DIV_0|ADC10SSEL_0|CONSEQ_0;
	ADC10AE0 = BIT1;				// Enable analog input on channel 1
// Timer_A from ACLK, no division, continous mode, clear, no interrupts
	TACTL = TASSEL_1 | ID_0 | MC_2 | TACLR;
	for (;;) {						// Loop forever taking measurements
		__low_power_mode_3();		// LPM3 between interrupts
	}
}
//----------------------------------------------------------------------
// ISR for watchdog: enable potential divider, set up delay for charging
//----------------------------------------------------------------------
#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR (void)		// Acknowledged automatically
{
	LED ^= 1;						// Flash LED to show activity
	POTDIV = 1;						// Apply VCC to potential divider
	do {							// Loop until two successive reads
		TACCR0 = TAR;				//   of TAR agree with each other
	} while (TACCR0 != TAR);
	TACCR0 += CHARGING;				// Delay for charging pot div
	TACCTL0 = CCIE;					// Enable interrupts, clear any flag
}
//----------------------------------------------------------------------
// Delay here to allow potential divider to recharge
//----------------------------------------------------------------------
// Interrupt service routine for CCIFG0: start new reading
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)	// Acknowledged automatically
{
	TACCTL0 = 0;					// Disable further CCIFG0 interrupts
	ADC10CTL0_bit.ADC10ON = 1;		// Turn on ADC
	ADC10CTL0 |= (ENC|ADC10SC);		// Enable ADC10, start conversion
}
//----------------------------------------------------------------------
// Wait here for conversion to complete
//----------------------------------------------------------------------
// Interrupt service routine for ADC10: disable ADC, update LED
//----------------------------------------------------------------------
#pragma vector = ADC10_VECTOR
__interrupt void ADC10_ISR (void)	// Acknowledged automatically
{
	POTDIV = 0;						// Remove VCC from potential divider
	ADC10CTL0_bit.ENC = 0;			// Disable conversions
	ADC10CTL0_bit.ADC10ON = 0;		// Turn off ADC
	if (((int16_t) ADC10MEM) > 0) {	// Is output positive?
		LED = 1;					// Yes: Turn LED on
	} else {
		LED = 0;					// No: Turn LED off
	}
}
