// ad10led5.c - measure input voltage from pot divider using ADC10
// Average a block of readings accumulated using DTC
// ADC10 default refs (VSS, VCC), internal clock, interrupt after block
// Home-made analog demo board for MSP430F20xx:
//   10k on P1.5, 20k variable on P1.6, junction to P1.1 with 10nF cap
//   LED active high on P2.6
// Default DCO, no crystal, no ACLK, power from JTAG
// J H Davies, 2007-09-10; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x2.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

#define	LED		P2OUT_bit.P2OUT_6	// Output bit for LED
#define	NSAMPLES	16				// Number of samples in each block

void main (void)
{
	uint16_t samples[NSAMPLES];		// Block of samples from ADC10
	uint32_t average;				// Average value of block
	uint16_t i;						// Loop counter

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	P2SEL = 0;						// Digital i/o rather than crystal
	P2OUT = 0;
	P2DIR = BIT6|BIT7;				// Unused outputs driven low
	P1OUT = BIT5;					// VCC to 10k fixed resistor
	P1DIR = BIT3|BIT5|BIT6|BIT7;	// Unused outputs driven low
// VCC and VSS refs, sample 4 cycs, multiple conversions, interrupts
	ADC10CTL0 = SREF_0 | ADC10SHT_0 | MSC | ADC10ON | ADC10IE;
// Input channel 1, trigger using ADC10SC bit, simple binary,
//   internal osc undivided, single channel repeat conversions
	ADC10CTL1 = INCH_1 | SHS_0 | ADC10DIV_0 | ADC10SSEL_0 | CONSEQ_2;
	ADC10AE0 = BIT1;				// Enable analog input on channel 1
	ADC10DTC0 = 0;					// Transfer single block (default)
	ADC10DTC1 = NSAMPLES;			// Number of readings in block
	for (;;) {						// Loop forever taking measurements
		ADC10SA = (unsigned short) samples; // Start address, enable DTC
		ADC10CTL0 |= (ENC|ADC10SC);	// Enable ADC10, start conversions
		__low_power_mode_0();		// Wait for batch to complete
		ADC10CTL0_bit.ENC = 0;		// Disable further ADC10 conversions
		average = NSAMPLES/2;		// To reduce bias when dividing
		for (i = 0; i < NSAMPLES; ++i) {
			average += samples[i];	// Accumulate sum
		}
		average /= NSAMPLES;		// Arithmetic mean of samples
		if (average > BIT9) {		// Is output in top half of range?
			LED = 1;				// Yes: Turn LED on
		} else {
			LED = 0;				// No: Turn LED off
		}
	}
}
//----------------------------------------------------------------------
// ISR for ADC10 at end of block: return to active mode
//----------------------------------------------------------------------
#pragma vector = ADC10_VECTOR
__interrupt void ADC10_ISR (void)	// Acknowledged automatically
{
	__low_power_mode_off_on_exit();	// Return to active mode on exit
}
