// eZ430led2.c - self-dimming LED on P1.0 in eZ430 using SD16A
// PWM controlled by software, about 100Hz from ACLK = VLO,
//   SD16A measures light during off phase of PWM
// Calibrated 8MHz DCO, no crystal, ACLK from VLO, power from JTAG (SBW)
// J H Davies, 2007-09-30; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

#define	LED_OUT		P1OUT_bit.P1OUT_0	// Output to LED on P1.0
#define	LED_ANALOG	SD16AE_bit.SD16AE0	// Enable analog input from LED
#define	RANGE		4096			// Dynamic range of values from SD16
#define PWM_MAX		128				// Maximum value of duty cycle
#define	DIVISOR		(RANGE/PWM_MAX)	// For converting SD16 -> PWM
#define	SENSE_TIME	2				// Cycles of TACLK needed for SD16

uint16_t dutyCycle = PWM_MAX;		// Duty cycle computed from SD16
									// Start at maximum (but LED off)
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	BCSCTL1 = CALBC1_8MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_8MHZ;			// Calibrated tap and modulation
	BCSCTL2 = DIVS_3;				// SMCLK = DCO / 8 = 1MHz
	BCSCTL3 = LFXT1S_2;				// Select ACLK from VLO (no crystal)
	P2SEL = 0;						// Digital i/o rather than crystal
	P2REN = BIT6|BIT7;				// Pull Rs on unused pins (6 and 7)
	P1REN = ~BIT0;					// Pull Rs on all pins except 0
	P1DIR = BIT0;					// To drive LED on P1.0
	LED_OUT = 0;					// LED initially off (active high)
// Configure SD16A: clock from SMCLK, no division, internal reference on
	SD16CTL = SD16XDIV_0 | SD16DIV_0 | SD16SSEL_1 | SD16REFON;
// Unipolar, single convs, OSR = 32, interrupts on finish
	SD16CCTL0 = SD16UNI | SD16SNGL | SD16OSR_32 | SD16IE;
// PGA gain = 16, input channel A0+/-, result after 4th conversion
	SD16INCTL0 = SD16GAIN_16 | SD16INCH_0 | SD16INTDLY_0;
// Timer_A for software-assisted PWM using channel 1, up to TACCR0 mode
	TACCR0 = PWM_MAX + SENSE_TIME;	// Overall period
	TACCR1 = dutyCycle;				// Initial duty cycle
	TACCTL1 = CCIE;					// Interrupts on compare
// Start Timer_A from ACLK, undivided, up mode, clear, interrupts
	TACTL = TASSEL_1 | ID_0 | MC_1 | TACLR | TAIE;
	for (;;) {						// Loop forever
		__low_power_mode_3();		// All action in interrupts
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for CCIFG1 and TAIFG; share vector
//----------------------------------------------------------------------
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// Shared ISR for CCIFG1 and TAIFG
{
	switch (__even_in_range(TAIV, TAIV_TAIFG)) {	// Acknowledges int
	case 0:						// No interrupt pending
		break;					// No action
	case TAIV_TAIFG:			// TAIFG vector
// Start PWM duty cycle by turning LED on and updating duty cycle
		LED_OUT = 1;			// Turn LED on; duty cycle always > 0
		TACCR1 = dutyCycle;		// Update duty cycle from SD16 reading
		break;
	case TAIV_CCIFG1:			// CCIFG1 vector
// Finish PWM duty cycle by turning off LED, then measuring light level
		LED_OUT = 0;			// End of duty cycle: Turn off LED
		LED_ANALOG = 1;			// Switch LED to SD16A input A0+
		SD16CCTL0_bit.SD16SC = 1;	// Start SD16A conversion
// Change mode from LPM3 to LPM0 on exit to provide SMCLK for SD16
		__bic_SR_register_on_exit(LPM3_bits);
		__bis_SR_register_on_exit(LPM0_bits);
		break;
	default:					// Should not be possible: ignore
		break;
	}
}
//----------------------------------------------------------------------
// ISR for SD16A: compute new duty cycle in range [1, PWM_MAX]
//----------------------------------------------------------------------
#pragma vector = SD16_VECTOR
__interrupt void SD16_ISR (void)	// Acknowledged when SD16MEM0 read
{
	static uint16_t floor = 0xFFFF - RANGE;	// Dark reading from SD16

	LED_ANALOG = 0;				// Switch LED back to digital output
	if (SD16MEM0 < floor) {		// Update floor if new reading is lower
		floor = SD16MEM0;
		dutyCycle = 1;			// Minimum value; never go down to 0
	} else if (SD16MEM0 >= (floor + RANGE - DIVISOR)) {
		dutyCycle = PWM_MAX;	// Maximum value (saturates)
	} else {
		dutyCycle = (SD16MEM0 - floor) / DIVISOR + 1;
	}
// Change mode from LPM0 to LPM3 on exit: SMCLK no longer needed
	__bic_SR_register_on_exit(LPM0_bits);	// (not really necessary)
	__bis_SR_register_on_exit(LPM3_bits);
}
