// exbdtmp1.c - Display temperature to 0.1 deg C using ADC12
// Measurements at 1s intervals set by basic timer, run by interrupts
// Delay for stabilizing reference set by watchdog interval timer
// TI Experimenter's Board with F4619, default clocks
// J H Davies, 2007-09-12; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
#include "LCDutils2.h"				// TI exp board utility functions

void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	IE1_bit.WDTIE = 1;				// Enable WDT interval interrupts
	FLL_CTL0 = XCAP14PF;			// Configure load caps (14pF)
	PortsInit();					// Initialize ports
	LCDInit();						// Initialize SBLCDA4
	DisplayHello();					// Display HELLO on LCD
// Set up ADC12 but leave both ADC and reference off
	ADC12CTL0 = SHT0_8;				// Sample 256 cycles, Vref+ = 1.5V
// Start with conversion 0, trigger from ADC12SC, use sample timer,
//   ADC12OSC undivided, single channel single conversion
	ADC12CTL1 = CSTARTADD_0|SHS_0|SHP|ADC12DIV_0|ADC12SSEL_0|CONSEQ_0;
// Conversion memory 0: end of seq, Vref and VSS refs, input channel 10
	ADC12MCTL0 = EOS | SREF_1 | INCH_10;
	ADC12IE = BIT0;					// Enable interrupts from memory 0
// Initialize basic timer: ACLK, cascaded, 1s interrupts, stopped
	BTCTL = BTHOLD | BTDIV | BT_fCLK2_DIV128;
	BTCNT1 = 0;						// Clear counters
	BTCNT2 = 0;
	IE2_bit.BTIE = 1;				// Enable basic timer interrupts
	BTCTL &= ~BTHOLD;				// Start basic timer running
	for (;;) {						// Measurements triggered by ints
		__low_power_mode_3();		// LPM3 between interrupts
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for basic timer, requested every 1s
// Enable ADC12 reference, allow time to stabilize using watchdog timer
//----------------------------------------------------------------------
#pragma vector = BASICTIMER_VECTOR
__interrupt void BASICTIMER_ISR (void)	// Acknowledged automatically
{
	ADC12CTL0_bit.REFON = 1;		// Turn ADC12 reference on
// Start watchdog as interval timer, 512 cycles of ACLK, clear
	WDTCTL = WDTPW | WDTTMSEL | WDTCNTCL | WDTSSEL | WDTIS1;
}
//----------------------------------------------------------------------
// Wait for reference voltage to stabilize
//----------------------------------------------------------------------
// ISR for watchdog interval timer: start ADC12 conversion
//----------------------------------------------------------------------
#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR (void)		// Acknowledged automatically
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	ADC12CTL0_bit.ADC12ON = 1;		// Turn ADC12 on
	ADC12CTL0 |= (ENC|ADC12SC);		// Enable, start single conversion
}
//----------------------------------------------------------------------
// Wait for ADC12 to complete conversion
//----------------------------------------------------------------------
// Interrupt service routine for ADC12, called when conversion complete
// Calculate temperature in tenths of celsius
//----------------------------------------------------------------------
#pragma vector = ADC12_VECTOR
__interrupt void ADC12_ISR (void)	// Flag NOT cleared automatically!
{
	uint32_t temperature;			// Converted value of temperature

	ADC12CTL0_bit.ENC = 0;			// Disable ADC12 before turning off
	ADC12CTL0_bit.ADC12ON = 0;		// Turn ADC12 off
	ADC12CTL0_bit.REFON = 0;		// Turn ADC12 reference off
	if (ADC12IV != 0x0006) {		// Should never happen!
//		for (;;) {					// Loop here forever as diagnostic
//		}
	} else {						// New temperature reading available
		temperature = ADC12MEM0;	// Raw converted value
		temperature *= 4200;		// Scaling factor (Vref/temp coeff)
		temperature += 2048;		// Allow for rounding in division
		temperature /= 4096;		// Divide by range of ADC10
		temperature -= 2780;		// Subtract offset to give celsius
		DisplayDeciCels ((int16_t) temperature);	// Display on LCD
	}
}
