// sd16dif2.c - lights LED for positive input voltage using SD16A
// SD16A channel A4+/- on P1.1/2, interrupt after continuous conversions
// Subtract PGA offset and average over block of readings
// Home-made analog demo board for MSP430F20xx:
//   10k on P1.5, 20k variable on P1.6, junction to P1.1 with 10nF cap
//   3 x 100k resistor network from VSS, VCC and P1.7, junction to P1.2
//   100n cap on P1.3 for Vref, LED active high on P2.6
// Calibrated 1MHz DCO, no crystal, no ACLK, power from JTAG
// J H Davies, 2007-10-03; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

#define	LED		P2OUT_bit.P2OUT_6	// Output bit for LED
#define	NSAMPLES	98				// Number of samples to average

void main (void)
{
	int16_t offset, difference;		// Data from SD16_A
	int32_t average;				// To accumulate average value
	uint16_t i;						// Loop counter

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	BCSCTL1 = CALBC1_1MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_1MHZ;			// Calibrated tap and modulation
	P2SEL = 0;						// Digital i/o rather than crystal
	P2OUT = 0;
	P2DIR = BIT6|BIT7;				// Unused outputs driven low
	P1OUT = BIT5;					// VCC to 10k fixed resistor
	P1DIR = BIT3|BIT5|BIT6|BIT7;	// Other outputs driven low
	P1SEL = BIT3;					// Enable Vref output to capacitor
// Configure SD16A: clock from SMCLK, no division, internal reference on
	SD16CTL = SD16XDIV_0 | SD16DIV_0 | SD16SSEL_1 | SD16REFON;
// OSR = 1024, bipolar 2s comp, continuous convs (default), interrupts
	SD16CCTL0 = SD16OSR_1024 | SD16DF | SD16IE;
	SD16AE = SD16AE1 | SD16AE2;		// Enable P1.1/2 for A4+/-
	for (;;) {						// Loop forever taking measurements
// PGA = 32, shorted input A7 to measure PGA offset, interrupts after 4
		SD16INCTL0 = SD16GAIN_32 | SD16INCH_7 | SD16INTDLY_0;
		SD16CCTL0_bit.SD16SC = 1;	// Start new conversions
		__low_power_mode_0();		// Wait for first result
		offset = (int16_t) SD16MEM0;	// Store input offset
		SD16CCTL0_bit.SD16SC = 0;	// Halt conversions
		average = NSAMPLES/2;		// Reduce bias in division
// PGA gain = 32, external input A4+/-, interrupts after 4th conversion
		SD16INCTL0 = SD16GAIN_32 | SD16INCH_4 | SD16INTDLY_0;
		SD16CCTL0_bit.SD16SC = 1;	// Start new conversions
		for (i = 0; i < NSAMPLES; ++i) {
			__low_power_mode_0();		// Wait for next result
			average += (int16_t) SD16MEM0;	// Accumulate sum
		}
		SD16CCTL0_bit.SD16SC = 0;	// Halt conversions
		average /= NSAMPLES;		// Calculate mean
		difference = ((int16_t) average) - offset;	// Subtract offset
		if (difference > 0) {		// Is corrected output positive?
			LED = 1;				// Yes: Turn LED on
		} else {
			LED = 0;				// No: Turn LED off
		}
	}
}
//----------------------------------------------------------------------
// ISR for SD16A: clear flag in software (do not read SD16MEM0 here)
//----------------------------------------------------------------------
#pragma vector = SD16_VECTOR
__interrupt void SD16_ISR (void)	// NOT acknowledged automatically
{
	SD16CCTL0_bit.SD16IFG = 0;		// Acknowledge interrupt, clear flag
	__low_power_mode_off_on_exit();	// Return to active mode
}
