// sd16led3.c - lights LED for high input voltage using SD16A
// SD16A channel A4+ on P1.1, A4- int gnd, "single" convs (actually 4)
// Store a block of readings for statistical analysis
// Home-made analog demo board for MSP430F20xx:
//   10k on P1.5, 20k variable on P1.6, junction to P1.1 with 10nF cap
//   100n cap on P1.3 for Vref, LED active high on P2.6
// Calibrated 1MHz DCO, no crystal, no ACLK, power from JTAG
// J H Davies, 2007-10-08; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

#define	LED		P2OUT_bit.P2OUT_6	// Output bit for LED
#define	NSAMPLES	32				// Number of samples in each block
__no_init uint16_t samples[NSAMPLES];	// Block of samples from SD16A

void main (void)
{
	uint32_t average;				// Average value of block
	volatile uint16_t i;			// Loop counter

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	BCSCTL1 = CALBC1_1MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_1MHZ;			// Calibrated tap and modulation
	P2SEL = 0;						// Digital i/o rather than crystal
	P2OUT = 0;
	P2DIR = BIT6|BIT7;				// Unused outputs driven low
	P1OUT = BIT5;					// VCC to 10k fixed resistor
	P1DIR = BIT3|BIT5|BIT6|BIT7;	// Unused outputs driven low
	P1SEL = BIT3;					// Enable Vref output to capacitor
// Configure SD16A: clock from SMCLK, no division, internal reference on
	SD16CTL = SD16XDIV_0 | SD16DIV_0 | SD16SSEL_1 | SD16REFON;
// Unipolar output, "single" convs, OSR = 256, interrupts on finish
	SD16CCTL0 = SD16UNI | SD16SNGL | SD16OSR_256 | SD16IE;
// PGA gain = 1, inpt channel A4+/-, interrupt and stop after 4th result
	SD16INCTL0 = SD16GAIN_1 | SD16INCH_4 | SD16INTDLY_0;
// Enable only P1.1 for A4+; leaves A4- connected internally to VSS
	SD16AE = SD16AE1;
	for (i = 0xFFFF; i > 0; --i) {	// Delay to allow Vref to stabilize
	}
	for (;;) {						// Loop forever
		for (i = 0; i < NSAMPLES; ++i) {	// Loop taking measurements
			SD16CCTL0_bit.SD16SC = 1;	// Trigger single conversion
			__low_power_mode_0();	// Sleep during conversion
			samples[i] = SD16MEM0;	// Store new data for analysis
		}
		average = NSAMPLES/2;		// To reduce bias when dividing
		for (i = 0; i < NSAMPLES; ++i) {
			average+= samples[i];	// Accumulate sum
		}
		average /= NSAMPLES;		// Arithmetic mean of samples
		if (average >= BITF) {		// Is average in top half of range?
			LED = 1;				// Yes: Turn LED on
		} else {
			LED = 0;				// No: Turn LED off
		}
	}
}
//----------------------------------------------------------------------
// Minimal ISR for SD16A: clear flag and return in active mode
//----------------------------------------------------------------------
#pragma vector = SD16_VECTOR
__interrupt void SD16_ISR (void)	// NOT acknowledged automatically
{
	SD16CCTL0_bit.SD16IFG = 0;		// Clear flag, acknowledge interrupt
	__low_power_mode_off_on_exit();	// Return to active mode
}
