// trans1.c - measure RC ch and discharge transient with Comparator_A+
// Timer_A channel 0 for output, channel 1 for comparator
// Home-made analog demo board for MSP430F20xx:
//	 10k resistor driven from P1.5/OUT0 to capacitor
//   10n capacitor input to P1.1/CA1, routed to CA+
//   Internal reference 0.5*VCC routed to CA-
//   Internal connection of CAOUT to CCI1B on Timer_A
// Calibrated 8MHz DCO, no xtl, no ACLK, power from JTAG
// J H Davies, 2007-08-20; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x20x1.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types
// TA1 captures, comparator (CCI1B), synchronized, interrupt, disabled
#define COMP_CAP		(CCIS_1 | SCS | CAP | CCIE)

void main (void)
{
	uint16_t ChargeTime, DischargeTime;	// Measured transient times

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	BCSCTL1 = CALBC1_8MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_8MHZ;			// Calibrated tap and modulation
	P2SEL = 0;						// Digital i/o rather than crystal
	P2OUT = 0;						// Unused outputs driven low
	P2DIR = BIT6|BIT7;
	P1OUT = 0;						// Unused pins are output driven low
	P1DIR = BIT3|BIT5|BIT7;			//   (Comparator_A+ overrides this)
	P1SEL = BIT5;					// Timer_A ch 0 output to resistor
// Set up input, output and ref of Comparator_A+ (leave it off for now)
	CACTL1 = CARSEL | CAREF_2;		// 0.5*VCC reference to CA- input
	CACTL2 = P2CA4 | CAF;			// CA1 to CA+ input, output filtered
// Diagnostic: CA3 to CA- terminal to check reference externally
//	CACTL2 = P2CA4 | P2CA2 | P2CA1 | CAF;
	CAPD = BIT1;					// Disable P1.1 digital i/o (CA1)
// Timer_A: SMCLK, no division, continuous, no need to clear, no ints
	TACTL = TASSEL_2 | ID_0 | MC_2;
	DischargeTime = 1000;			// Dummy value for first delay
	for (;;) {						// Loop forever taking measurements
		CACTL1_bit.CAON = 1;		// Comparator on
// Charging transient: discharge C fully, Set output, wait for Comp_A+
		TACCR0 = TAR + 10*DischargeTime;	// Delay for full discharge
		TACCTL0 = OUTMOD_1;			// Set on compare to start charge
		TACCTL1 = COMP_CAP | CM_1;	// Capture comparator rising
		__low_power_mode_0();		// Wait for timer and comparator
		ChargeTime = TACCR1 - TACCR0;	// Duration of charge
// Discharging transient: charge C fully, Reset output, wait for Comp_A+
		TACCR0 = TAR + 10*ChargeTime;	// Delay to allow full charge
		TACCTL0 = OUTMOD_5;			// Reset output to start discharge
		TACCTL1 = COMP_CAP | CM_2;	// Capture comparator falling
		__low_power_mode_0();		// Wait for timer and comparator
		DischargeTime = TACCR1 - TACCR0;	// Duration of discharge
// Diagnostic: look for effect of comparator offset voltage
//		CACTL1_bit.CAEX ^= 1;		// Exchange inputs to comparator
		CACTL1_bit.CAON = 0;		// Comparator off
// Do something useful with the measurements here!
	}
}
//----------------------------------------------------------------------
// ISR for CCIFG1: disable further captures and return to active mode
//----------------------------------------------------------------------
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// Shared ISR for CCIFG1 and TAIFG
{
	TACCTL1 = COMP_CAP | CM_0;		// Disable further caps, clear flags
	__low_power_mode_off_on_exit();	// Return to active mode on exit
}
