// usci2cmastsm1.c - Receive temperature over I2C using USCI_B0
// Master mode, receive two bytes from slave; needs pullups on SCL, SDA!
// State machine to control flow for I2C interrupts; display in main
// FG4619 on TI Experimenter's Board, 32KHz crystal, 1MHz DCO (default)
//   SCL on P3.2, SDA on P3.1
//   Display temperature on LCD, flash LED to show activity
// J H Davies, 2008-01-08; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Needs corrected version for USCI!
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
#include "LCDutils2.h"				// TI exp board utility functions

#define N_MESSAGE	2				// Number of bytes in message
uint8_t	RXcount;					// Counter for bytes
uint8_t RXdata[N_MESSAGE];			// Received data, [0] = LSB (last)
enum {
	I2CIdle,
	I2CSuccess,
	I2CNack
} I2CStatus = I2CIdle;				// Return status of I2C to main
#define SLAVE_ADDRESS	0x48		// I2C address of thermometer
#define LED		P5OUT_bit.P5OUT_1	// Output pin for LED (active high)
									// Pin is output low by default
void main (void)
{
	volatile uint16_t i;			// Loop counter to stabilize FLL+
	int16_t	Temperature;			// Value received over I2C

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	FLL_CTL0 = XCAP14PF;			// 14pF load caps; 1MHz default
	do {							// Wait until FLL has locked
		for (i = 0x2700; i > 0; --i) {	// One loop should be enough!
		}             				// Delay for FLL+ to lock
		IFG1_bit.OFIFG = 0;			// Attempt to clear osc fault flag
	} while (IFG1_bit.OFIFG != 0);	// Repeat if not yet clear
	PortsInit();					// Initialize ports
	LCDInit();						// Initialize SBLCDA4
	DisplayHello();					// Display HELLO on LCD
	P3SEL = BIT1 | BIT2;			// Route pins to USCI_B for I2C
// 7-bit addresses (default), single master, master mode, I2C, synch
	UCB0CTL0 = UCMST | UCMODE_3 | UCSYNC;
// Clock from SMCLK, receiver (default), hold in reset
	UCB0CTL1 = UCSSEL1 | UCSWRST;
	UCB0BR1 = 0;					// Upper byte of divider word
	UCB0BR0 = 10;					// Clock = SMCLK / 10 = 100kHz
	UCB0I2CSA = SLAVE_ADDRESS;		// Slave to be addressed
	UCB0I2COA = 0;					// Ignore genl call; own address = 0
	UCB0I2CIE = UCNACKIE;			// Enable interrupts on Nack
	UCB0CTL1 &= ~UCSWRST;			// Release from reset
	IE2_bit.UCB0RXIE = 1;			// Enable interrupts on Receive
// Set up basic timer for interrupts at 2Hz (500ms)
	BTCTL = BTHOLD | BT_ADLY_500;	// Hold, period = 500ms from ACLK
	BTCNT2 = 0;						// Clear counter
	BTCTL &= ~BTHOLD;				// Start basic timer
	IE2_bit.BTIE = 1;				// Enable basic timer interrupts
	for (;;) {						// Transfers triggered by BT
		__low_power_mode_3();		// Wait for BT (needs only ACLK)
		switch (I2CStatus) {
		case I2CIdle:				// Should never happen!
			break;
		case I2CSuccess:			// Successful reception
			Temperature = (RXdata[1] << 8) | RXdata[0];	// Assemble word
			DisplayCentiCels (Temperature);	// Display temperature
			break;
		case I2CNack:				// Address not recognised
			DisplayErr();			// Display error message
			break;
		default:					// Should never happen!
			I2CStatus = I2CIdle;	// Emergency reset
			break;
		}
	}
}
//----------------------------------------------------------------------
// ISR for basic timer: start I2C transfer
//----------------------------------------------------------------------
#pragma vector = BASICTIMER_VECTOR
__interrupt void BASICTIMER_ISR (void)	// Acknowledged automatically
{
	LED = 1;						// Show start of activity
	RXcount = N_MESSAGE;			// Initialize byte counter for MSB
	UCB0CTL1 |= UCTXSTT;			// Send Start and slave address
}
//----------------------------------------------------------------------
// ISR for UCB0RXIFG: store received byte, send Stop condition, display
// Shared with UCA0TXIFG and UCB0TXIFG, neither active
//----------------------------------------------------------------------
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR (void)	// Acknowledge by read of RXBUF
{
	if (RXcount > 0) {					// Byte expected?
		RXdata[--RXcount] = UCB0RXBUF;	// Yes: Update ctr, store byte
		if (RXcount == 1) {				// Last byte but one
			UCB0CTL1 |= UCTXSTP;		// Send Stop cond after byte
		} else if (RXcount == 0) {		// Last byte
			I2CStatus = I2CSuccess;			// Tell main() of success
			__low_power_mode_off_on_exit();	// Return to main to display
			LED = 0;						// Finished!
		}
	} else {							// No: Should never happen
		IFG2_bit.UCB0RXIFG = 1;			// Clear flag to acknowledge
	}
}
//----------------------------------------------------------------------
// ISR for UCB0NACKIFG (not-acknowledge): abort transaction
// Shared with UCA0RXIFG and other UCB0 state-change flags, all inactive
//----------------------------------------------------------------------
#pragma vector = USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR (void)	// Not acknowledged automaticaly
{
	UCB0STAT &= ~UCNACKIFG;				// Clear Nack flag
	UCB0CTL1 |= UCTXSTP;				// Send Stop condition to finish
	I2CStatus = I2CNack;				// Tell main() of problem
	__low_power_mode_off_on_exit();		// Return to main for display
	LED = 0;							// Finished!
}
