// usci2cslvsm2.c - Receive temperature over I2C using USCI_B0
// Slave mode, receive two bytes from slave; needs pullups on SCL, SDA!
// Sends value from pushbuttons for delay if addressed as transmitter
// Control based on I2C interrupts: RXIFG, TXIFG, STTIFG, UCSTPIFG
// FG4619 on TI Experimenter's Board, 32KHz crystal, 1MHz DCO (default)
//   SCL on P3.2, SDA on P3.1
//   Display temperature on LCD, flash LED to show activity
// J H Davies, 2008-01-11; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Needs corrected version for USCI!
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
#include "LCDutils2.h"				// TI exp board utility functions

#define N_RECEIVE	2				// Number of bytes to receive
uint8_t	RXcount;					// Counter for bytes
uint8_t RXdata[N_RECEIVE];			// Received data, [0] = LSB (last)
#define OWN_ADDRESS		0x46		// I2C address of this device
#define LED		P5OUT_bit.P5OUT_1	// Output pin for LED (active high)
									// Pin is output low by default
void main (void)
{
	volatile uint16_t i;			// Loop counter to stabilize FLL+
	int16_t	Temperature;			// Value received over I2C

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	FLL_CTL0 = XCAP14PF;			// 14pF load caps; 1MHz default
	do {							// Wait until FLL has locked
		for (i = 0x2700; i > 0; --i) {	// One loop should be enough!
		}             				// Delay for FLL+ to lock
		IFG1_bit.OFIFG = 0;			// Attempt to clear osc fault flag
	} while (IFG1_bit.OFIFG != 0);	// Repeat if not yet clear
	PortsInit();					// Initialize ports
	LCDInit();						// Initialize SBLCDA4
	DisplayHello();					// Display HELLO on LCD
	P3SEL = BIT1 | BIT2;			// Route pins to USCI_B for I2C
// 7-bit addresses, single master, slave mode (all defaults), I2C, synch
	UCB0CTL0 = UCMODE_3 | UCSYNC;
// No need to configure UCB0CTL1, UCB0BR nor UCB0I2CSA for a slave
	UCB0I2COA = OWN_ADDRESS;		// Ignore genl call; own address
	UCB0CTL1 &= ~UCSWRST;			// Release from reset
	UCB0I2CIE = UCSTTIE | UCSTPIE;	// Enable interrupts on Start, Stop
	IE2 |= (UCB0RXIE|UCB0TXIE);		// Enable interrupts on RX and TX
	for (;;) {						// Transfers triggered by BT
		__low_power_mode_3();		// Need ACLK for LCD
		Temperature = (RXdata[1] << 8) | RXdata[0];	// Assemble word
		DisplayCentiCels (Temperature);	// Display temperature
	}
}
//----------------------------------------------------------------------
// ISR for UCSTTIFG (Start): start transaction, reset byte counter
//   UCSTPIFG (Stop): return to main() to display temperature
// Shared with UCA0RXIFG and other UCB0 state-change flags, all inactive
//----------------------------------------------------------------------
#pragma vector = USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR (void)	// Not acknowledged automaticaly
{
	if (UCB0STAT & UCSTTIFG) {			// Start condition?
		UCB0STAT &= ~UCSTTIFG;			// Yes: Clear start flag
		RXcount = N_RECEIVE;			// Initialize byte ctr for MSB
		LED = 1;						// Show start of transaction
	} else {							// Must be Stop condition
		UCB0STAT &= ~UCSTPIFG;			// Clear stop flag
		__low_power_mode_off_on_exit();	// Return to main, display temp
		LED = 0;						// Show end of transaction
	}
}
//----------------------------------------------------------------------
// ISR for UCB0RXIFG: store received byte; reading UCB0RXBUF clears flag
//   TXIFG: send value from pushbuttons; writing UCB0TXBUF clears flag
//   UCA0TXIFG: not active
//----------------------------------------------------------------------
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR (void)	// Not acknowledged automaticaly
{
	if (IFG2_bit.UCB0RXIFG) {			// Byte received? (MSB first)
		if (RXcount > 0) {				// Yes: Byte expected?
			RXdata[--RXcount] = UCB0RXBUF;	// Update ctr, store byte
		} else {						// Too many bytes, "impossible"
			IFG2_bit.UCB0RXIFG = 1;		// Clear flag to acknowledge
		}
	} else {							// No: Provide byte to transmit
		UCB0TXBUF = (~P1IN) & (BIT1|BIT0);	// Mask buttons, active low
	}
}
