// usciasync1.c - measure, send, receive, display temp'ture with USCI_A0
// 2 x 8-bit transfers, 9600 baud, 8-N-1 format, loopback
// FG4619 on TI Experimenter's Board, 32KHz crystal, 1MHz DCO (default),
//   UCA0TXD on P2.4, UCA0RXD on P2.5, temp from ADC12 every 1s with BT
// J H Davies, 2008-01-25; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
#include "LCDutils2.h"				// TI exp board utility functions

#define N_MESSAGE	2				// Number of bytes in message
uint8_t	RXcount = 0;				// Counters for bytes transferred
uint8_t	TXcount = 0;
uint8_t RXdata[N_MESSAGE], TXdata[N_MESSAGE];	// Recd and transd data
#define LED		P5OUT_bit.P5OUT_1	// Output pin for LED (active high)
									// Pin is output low by default
void main (void)
{
	volatile uint16_t i;			// Loop counter to stabilize FLL+

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	IE1_bit.WDTIE = 1;				// Enable WDT interval interrupts
	FLL_CTL0 = XCAP14PF;			// 14pF load caps, 1MHz default
	do {							// Wait until FLL has locked
		for (i = 0x2700; i > 0; --i) {	// One loop should be enough!
		}             				// Delay for FLL+ to lock
		IFG1_bit.OFIFG = 0;			// Attempt to clear osc fault flag
	} while (IFG1_bit.OFIFG != 0);	// Repeat if not yet clear
	PortsInit();					// Initialize ports
	LCDInit();						// Initialize SBLCDA4
	DisplayHello();					// Display HELLO on LCD
// Set up ADC12 but leave both ADC and reference off
	ADC12CTL0 = SHT0_8;				// Sample 256 cycles, Vref+ = 1.5V
// Start with conversion 0, trigger from ADC12SC, use sample timer,
//   ADC12OSC undivided, single channel single conversion
	ADC12CTL1 = CSTARTADD_0|SHS_0|SHP|ADC12DIV_0|ADC12SSEL_0|CONSEQ_0;
// Conversion memory 0: end of seq, Vref and VSS refs, input channel 10
	ADC12MCTL0 = EOS | SREF_1 | INCH_10;
	ADC12IE = BIT0;					// Enable interrupts from memory 0
// Set up Basic Timer1
	BTCTL = BTHOLD | BT_ADLY_1000;	// Hold, period = 1000ms from ACLK
	BTCNT2 = 0;						// Clear counter
	BTCTL &= ~BTHOLD;				// Start basic timer
	IE2_bit.BTIE = 1;				// Enable basic timer interrupts
// Set up USCI_A for asynchronous comms, 9600 baud using ACLK
	P2SEL = BIT4 | BIT5;			// Route pins to USCI_A for comms
// Defaults: no parity, lsb first, 8-bit data, 1 stop bit, UART, async
	UCA0CTL0 = 0;
	UCA0CTL1 = UCSSEL_1 | UCSWRST;	// Clock from ACLK; hold in reset
	UCA0BR1 = 0;					// Upper byte of divider word
	UCA0BR0 = 3;					// Divider from ACLK to BITCLK
	UCA0MCTL = UCBRS_3;				// Low-frequency mode, modul'n = 3
	UCA0STAT = UCLISTEN;			// Loopback mode	
	UCA0CTL1 &= ~UCSWRST;			// Release from reset
	IE2_bit.UCA0RXIE = 1;			// Enable interrupts on RX all time
	for (;;) {						// Transmissions triggered by BT
		__low_power_mode_3();
	}
}
//----------------------------------------------------------------------
// ISR for USCI_A,B0 TX: load next byte for TX, halt interrupts at end
//----------------------------------------------------------------------
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR (void)	// Acknowledge by write to TXBUF
{
	UCA0TXBUF = TXdata[TXcount++];	// Send next byte, update counter
	if (TXcount >= N_MESSAGE) {		// Sent complete message?
		IE2_bit.UCA0TXIE = 0;		// Disable further interrupts
	}
}
//----------------------------------------------------------------------
// ISR for USCI_A,B0 RX: store data, process when all bytes received
//----------------------------------------------------------------------
#pragma vector = USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR (void)	// Acknowledge by read of RXBUF
{
	int16_t	Temperature;			// Value received over SPI
	
	RXdata[RXcount++] = UCA0RXBUF;	// Store recd data, update counter
	if (RXcount >= N_MESSAGE) {		// Received complete message?
		Temperature = (RXdata[1] << 8) | RXdata[0];	// Assemble word
		DisplayDeciCels (Temperature);	// Display temperature to 0.1oC
		RXcount = 0;				// Reset byte counter
		LED = 0;					// Show end of activity
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for basic timer, requested every 1s
// Enable ADC12 reference, allow time to stabilize using watchdog timer
//----------------------------------------------------------------------
#pragma vector = BASICTIMER_VECTOR
__interrupt void BASICTIMER_ISR (void)	// Acknowledged automatically
{
	LED = 1;						// Show start of activity
	ADC12CTL0_bit.REFON = 1;		// Turn ADC12 reference on
// Start watchdog as interval timer, 512 cycles of ACLK, clear
	WDTCTL = WDTPW | WDTTMSEL | WDTCNTCL | WDTSSEL | WDTIS1;
}
//----------------------------------------------------------------------
// Wait for reference voltage to stabilize
//----------------------------------------------------------------------
// ISR for watchdog interval timer: start ADC12 conversion
//----------------------------------------------------------------------
#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR (void)		// Acknowledged automatically
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	ADC12CTL0_bit.ADC12ON = 1;		// Turn ADC12 on
	ADC12CTL0 |= (ENC|ADC12SC);		// Enable, start single conversion
}
//----------------------------------------------------------------------
// Wait for ADC12 to complete conversion
//----------------------------------------------------------------------
// Interrupt service routine for ADC12, called when conversion complete
// Calculate temperature in tenths of celsius
//----------------------------------------------------------------------
#pragma vector = ADC12_VECTOR
__interrupt void ADC12_ISR (void)	// Flag NOT cleared automatically!
{
	uint32_t temperature;			// Converted value of temperature

	ADC12CTL0_bit.ENC = 0;			// Disable ADC12 before turning off
	ADC12CTL0_bit.ADC12ON = 0;		// Turn ADC12 off
	ADC12CTL0_bit.REFON = 0;		// Turn ADC12 reference off
	if (ADC12IV == 0x0006) {		// No other value should happen!
		temperature = ADC12MEM0;	// Raw converted value
		temperature *= 4200;		// Scaling factor (Vref/temp coeff)
		temperature += 2048;		// Allow for rounding in division
		temperature /= 4096;		// Divide by range of ADC10
		temperature -= 2780;		// Subtract offset to give celsius
		TXdata[0] = (uint8_t) temperature;			// Extract LSB
		TXdata[1] = (uint8_t) (temperature >> 8);	//   and MSB
		TXcount = 0;				// Initialize byte counter
		IE2_bit.UCA0TXIE = 1;		// Enable interrupts to start TX
	}
}
