// uscispimaster1.c - receive temperature over SPI using USCI_B
// 2 x 8-bit transfers, SPI mode 0 (CPOL = CPHA = 0), master mode
// FG4619 on TI Experimenter's Board, 32KHz crystal, 1MHz DCO (default)
//   SCLK on P3.3, MOSI on P3.1, MISO on P3.2
//   Display temperature on LCD, flash LED to show activity
//   P3.0 used as _SS (nSS) output in software before word, not bytes
// J H Davies, 2007-11-11; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
#include "LCDutils2.h"				// TI exp board utility functions

#define N_MESSAGE	2				// Number of bytes in message
uint8_t	RXcount, TXcount;			// Counters for bytes
uint8_t RXdata[N_MESSAGE], TXdata[N_MESSAGE];	// Recd and transd data
#define nSS		P3OUT_bit.P3OUT_0	// Output pin for _SS (active low)
#define LED		P5OUT_bit.P5OUT_1	// Output pin for LED (active high)
									// Pin is output low by default
void main (void)
{
	volatile uint16_t i;			// Loop counter to stabilize FLL+
	int16_t	Temperature;			// Value received over SPI

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	FLL_CTL0 = XCAP14PF;			// 14pF load caps; 1MHz default
	do {							// Wait until FLL has locked
		for (i = 0x2700; i > 0; --i) {	// One loop should be enough!
		}             				// Delay for FLL+ to lock
		IFG1_bit.OFIFG = 0;			// Attempt to clear osc fault flag
	} while (IFG1_bit.OFIFG != 0);	// Repeat if not yet clear
	PortsInit();					// Initialize ports
	LCDInit();						// Initialize SBLCDA4
	DisplayHello();					// Display HELLO on LCD
	P3OUT_bit.P3OUT_0 = 1;			// High for nSS inactive
	P3DIR_bit.P3DIR_0 = 1;			// Enable for nSS output
	P3SEL = BIT1 | BIT2 | BIT3;		// Route pins to USCI_B for SPI
// SPI mode 0 needs CPOL=CKPL=0, CPHA=0 -> CKPH=1; msb first, master,
//   8 bit (default), 3-wire (default, mode 0), synchronous
	UCB0CTL0 = UCCKPH | UCMSB | UCMST | UCMODE_0 | UCSYNC;
	UCB0CTL1 = UCSSEL1 | UCSWRST;	// Clock from SMCLK; hold in reset
	UCB0BR1 = 0;					// Upper byte of divider word
	UCB0BR0 = 10;					// Clock = SMCLK / 10 = 100kHz
	UCB0CTL1 &= ~UCSWRST;			// Release from reset
	IE2_bit.UCB0RXIE = 1;			// Enable interrupts on RX all time
	BTCTL = BTHOLD | BT_ADLY_500;	// Hold, period = 500ms from ACLK
	BTCNT2 = 0;						// Clear counter
	BTCTL &= ~BTHOLD;				// Start basic timer
	IE2_bit.BTIE = 1;				// Enable basic timer interrupts
	for (;;) {						// Transmissions triggered by BT
		__low_power_mode_3();		// Wait for BT (needs only ACLK)
// Set up SPI transaction, to be performed using interrupts
		LED = 1;					// Show start of activity
		for (i = 0; i < N_MESSAGE; ++i) {
			TXdata[i] = 0xAA;		// Make outgoing message
		}							//   (rather boring here!)
		RXcount = TXcount = 0;		// Initialize message byte counters
		nSS = 0;					// Lower nSS (make active)
		IE2_bit.UCB0TXIE = 1;		// Enable TX interrupts, starts SPI
		__low_power_mode_0();		// Wait for SPI (needs SMCLK)
// SPI transaction completed		(but USCI would keep SMCLK active)
		nSS = 1;					// Raise nSS (make inactive)
		Temperature = (RXdata[0] << 8) | RXdata[1];	// Assemble word
		DisplayCentiCels (Temperature);	// Display temperature to 0.01oC
		LED = 0;					// Show end of activity
	}
}
//----------------------------------------------------------------------
// ISR for basic timer: return to main routine
//----------------------------------------------------------------------
#pragma vector = BASICTIMER_VECTOR
__interrupt void BASICTIMER_ISR (void)	// Acknowledged automatically
{
	__low_power_mode_off_on_exit();	// Return to start new SPI message
}
//----------------------------------------------------------------------
// ISR for USCI_A,B0 TX: load next byte for TX, halt interrupts at end
//----------------------------------------------------------------------
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR (void)	// Acknowledge by write to TXBUF
{
	UCB0TXBUF = TXdata[TXcount++];	// Send next byte, update counter
	if (TXcount >= N_MESSAGE) {		// Sent complete message?
		IE2_bit.UCB0TXIE = 0;		// Disable further interrupts
	}
}
//----------------------------------------------------------------------
// ISR for USCI_A,B0 RX: store data, return when all bytes received
//----------------------------------------------------------------------
#pragma vector = USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR (void)	// Acknowledge by read of RXBUF
{
	RXdata[RXcount++] = UCB0RXBUF;	// Store recd data, update counter
	if (RXcount >= N_MESSAGE) {		// Received complete message?
		__low_power_mode_off_on_exit();	// Return to process message
	}
}
