// uscispiloop1.c - receive temperature over SPI using USCI_B
// Send next delay value from buttons on P1.0,1 (active low)
// 2 x 8-bit transfers, SPI mode 3 (CPOL = CPHA = 1), slave mode
// FG4619 on TI Experimenter's Board, 32KHz crystal, 1MHz DCO (default)
//   SCLK on P3.3, MOSI on P3.1, MISO on P3.2
//   Display temperature on LCD, flash LED to show activity
// J H Davies, 2007-11-09; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430xG46x.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes
#include "LCDutils2.h"				// TI exp board utility functions

#define LED		P5OUT_bit.P5OUT_1	// Output pin for LED (active high)
									// Pin is output low by default
void main (void)
{
	volatile uint16_t i;			// Loop counter to stabilize FLL+

	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	FLL_CTL0 = XCAP14PF;			// 14pF load caps, 1MHz default
	do {							// Wait until FLL has locked
		for (i = 0x2700; i > 0; --i) {	// One loop should be enough!
		}             				// Delay for FLL+ to lock
		IFG1_bit.OFIFG = 0;			// Attempt to clear osc fault flag
	} while (IFG1_bit.OFIFG != 0);	// Repeat if not yet clear
	PortsInit();					// Initialize ports
	LCDInit();						// Initialize SBLCDA4
	DisplayHello();					// Display HELLO on LCD
	P3SEL = BIT1 | BIT2 | BIT3;		// Route pins to USCI_B for SPI
// SPI mode 3 needs CPOL=CKPL=1, CPHA=1 -> CKPH=0; msb first,
//   slave (default), 8 bit (default), 3-wire (default, mode 0), synch
	UCB0CTL0 = UCCKPL | UCMSB | UCMODE_0 | UCSYNC;
// No need to configure	UCB0CTL1, UCB0BR0,1 because clock not used
	UCB0CTL1 &= ~UCSWRST;			// Release from reset
	IE2 |= (UCB0TXIE|UCB0RXIE);		// Enable interrupts on RX and TX
	for (;;) {						// Transmissions triggered by master
		__low_power_mode_3();		// LPM3 between interrupts
	}
}
//----------------------------------------------------------------------
// ISR for USCI_A,B0 TX: load buffer with delay value from pushbuttons
//----------------------------------------------------------------------
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR (void)	// Not acknowledged automaticaly
{
	UCB0TXBUF = (~P1IN) & (BIT1|BIT0);	// Mask buttons, active low
}									// Writing to UCB0TXBUF clears flag
//----------------------------------------------------------------------
// ISR for USCI_A,B0 RX: store received data, send to LCD when complete
//----------------------------------------------------------------------
#pragma vector = USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR (void)	// Not acknowledged automaticaly
{
	static int16_t RXdata;				// Received data, both bytes
	static uint8_t bytesReceived = 0;	// Count bytes in message

	if (bytesReceived == 0) {
		bytesReceived = 1;
		RXdata = UCB0RXBUF << 8;	// Store received data, clears flag
		LED = 1;					// Mark first byte
	} else {
		bytesReceived = 0;
		RXdata |= UCB0RXBUF;		// Merge received data, clears flag
		DisplayCentiCels (RXdata);	// Display temperature to 0.01oC
		LED = 0;					// Mark end of transmission
	}
}
