// usi2cslave7.c - transmit temperature, I2C slave, optional clk stretch
// 2 bytes, MSB first, new reading after transmission; Vref kept active
// Simple control flow for I2C, mostly in main routine, Start interrupts
// F2013 on TI Experimenter's Board:
//   SCL on P1.6, SDA on P1.7, LED on P1.0 to show activity
// Calibrated 12MHz DCO, no crystal, no ACLK
// J H Davies, 2007-12-16; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes

#define LED		P1OUT_bit.P1OUT_0	// Output pin for LED (active high)
#define SCL		P1IN_bit.P1IN_6		// I/O pin for SCL (I2C clock)
#define SLAVE_ADDRESS	0x48		// I2C address of this thermometer
#define READ_ADDRESS 	((SLAVE_ADDRESS << 1) | BIT0)
									// Address plus bit R/_W = 1
void main (void)
{
	uint8_t  tempMSB, tempLSB;		// Temperature bytes to transmit
	uint32_t temperature;			// Converted value of temperature
									//   in units of 0.01degC
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	BCSCTL1 = CALBC1_12MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_12MHZ;			// Calibrated tap and modulation
	P2SEL = 0;						// Digital i/o rather than crystal
	P2DIR = 0;						// Inputs
	P2REN = BIT6 | BIT7;			// Pull resistors on unused pins
// USI will override these settings for SCL, SDA
	P1OUT = ~BIT0;					// LED low (off) others high
	P1DIR = BIT0;					// LED output, others input
	P1REN = ~BIT0;					// Pull unused pins up
// Configure SD16A: clock from SMCLK/12 = 1MHz, internal reference on
	SD16CTL = SD16XDIV_1 | SD16DIV_2 | SD16SSEL_1 | SD16REFON;
// Unipolar output, "single" convs, OSR = 1024, interrupts on finish
	SD16CCTL0 = SD16UNI | SD16SNGL | SD16OSR_1024 | SD16IE;
// PGA gain = 1, temperature sensor A6+/-, interrupts after 4th result
	SD16INCTL0 = SD16GAIN_1 | SD16INCH_6 | SD16INTDLY_0;
// No external inputs
	SD16AE = 0;
// Enable SDA and SCL, hold USI in reset state
// Defaults: msb first, slave, enable output latch, disable data output
	USICTL0 = USIPE7 | USIPE6 | USISWRST;
// I2C not SPI, can't clr USIIFG; Defaults: CKPH = 0 (CPHA = 1), no ints
	USICTL1 = USII2C | USIIFG;
//	No clock generator for slave, clock idles high (CKPL = CPOL = 1)
	USICKCTL = USICKPL;
	USICTL0 &= ~USISWRST;			// Release USI from reset
	for (;;) {						// Loop forever
// Clean up after previous transaction
		LED = 0;					// LED off while waiting for I2C
		USICNT = USISCLREL;			// Release SCL: we are not active
		USICTL1 &= ~USIIFG;			// Clr any pending flag (necessary?)
// Take new temperature reading with SD16
		SD16CCTL0_bit.SD16SC = 1;	// Trigger new conversion for temp
		__low_power_mode_0();		// LPM0 during conversion
		temperature = SD16MEM0;		// Raw converted value
		temperature *= 45454;		// Scaling factor (Vref/temp coeff)
		temperature += 0x8000;		// Allow for rounding in division
		temperature /= 0x10000;		// Divide by range of SD16A
		temperature -= 27300;		// Subtract offset to give celsius
//		temperature = 0xA5AA;		// Dummy value to transmit
		tempMSB = (uint8_t) (temperature >> 8);	// Extract bytes
		tempLSB = (uint8_t) temperature;		//   to transmit
// Data ready: activate interrupt to wait for Start condition
		USICTL1 &= ~USISTTIFG;		// Clr any pending Start interrupt
		USICTL1 |= USISTTIE;		// Enable Start interrupts
		__low_power_mode_4();		// Wait for Start interrupt, LPM4
// Start condition detected: USI now receiving address from master
		LED = 1;					// LED on while I2C active
		while ((USICTL1 & USIIFG) == 0) {
		}							// Wait for 8 bits to be received
// Check address: is it ours (with read direction, not write)?
		if (USISRL != READ_ADDRESS)	// Not us?
			continue;				// Abandon transaction: restart loop
// Send acknowledgement bit (low) after waiting for SCL to go low
		while (SCL == 1) {
		}							// Wait for SCL to go low
		USISRL = 0;					// Low bit to send
		USICTL0 |= USIOE;			// Enable output
		USICNT = 1;					// Send 1 bit, clr IFG, release SCL
		while ((USICTL1 & USIIFG) == 0) {
		}							// Wait for bit to be sent
// Send first byte of data (MSB of temperature) after waiting for SCL
		while (SCL == 1) {
		}							// Wait for SCL to go low
		USISRL = tempMSB;			// Send MSB of temperature
		USICNT = 8;					// Send 8 bits, clr IFG, release SCL
		while ((USICTL1 & USIIFG) == 0) {
		}							// Wait for bits to be sent
// Check for acknowledgment: receive Ack/Nack bit
		while (SCL == 1) {
		}							// Wait for SCL to go low
		USICTL0 &= ~USIOE;			// Disable output
		USICNT = 1;					// Rec 1 bit, clr IFG, release SCL
		while ((USICTL1 & USIIFG) == 0) {
		}							// Wait for bit to be received
		if ((USISRL & BIT0) != 0)	// Received Nack instead of Ack?
			continue;				// Abandon transaction: restart loop
// Acknowledgment received: send second (last) byte of data
		while (SCL == 1) {
		}							// Wait for SCL to go low
		USISRL = tempLSB;			// Send LSB of temperature
		USICTL0 |= USIOE;			// Enable output
		USICNT = 8;					// Send 8 bits, clr IFG, release SCL
		while ((USICTL1 & USIIFG) == 0) {
		}							// Wait for bits to be sent
// Receive final not-acknowledgment (and ignore it!)
		while (SCL == 1) {
		}							// Wait for SCL to go low
		USICTL0 &= ~USIOE;			// Disable output
		USICNT = 1;					// Rec 1 bit, clr IFG, release SCL
		while ((USICTL1 & USIIFG) == 0) {
		}							// Wait for bit to be received
	}								// Repeat loop (start with clean up)
}
//----------------------------------------------------------------------
// ISR for USI: Start condition, set up USI to receive address
//----------------------------------------------------------------------
#pragma vector = USI_VECTOR
__interrupt void USI_ISR (void)		// Not acknowledged automatically
{
	USICTL1 &= ~USISTTIE;			// Disable further Start interrupts
	USICNT = 8;						// Read 8 bits, clear USIIFG
	__low_power_mode_off_on_exit();	// Return to main function
	USICTL1 &= ~USISTTIFG;			// Acknowledge int'pt, release SCL
}
//----------------------------------------------------------------------
// ISR for SD16A: return to main function to compute temperature
//----------------------------------------------------------------------
#pragma vector = SD16_VECTOR
__interrupt void SD16_ISR (void)	// Not acknowledged automatically
{
	SD16CCTL0_bit.SD16IFG = 0;		// Clear flag, acknowledge interrupt
	__low_power_mode_off_on_exit();	// Return to main function
}
