// usi2cslvsm4.c - transmit temperature, I2C slave, clock stretching
// 2 bytes, MSB first, new reading after transmission; Vref kept active
// Simple state machine driven by USI interrupts, calculation in main
// F2013 on TI Experimenter's Board:
//   SCL on P1.6, SDA on P1.7, LED on P1.0 to show activity
// Calibrated 1MHz DCO, no crystal, no ACLK
// J H Davies, 2008-01-13; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes

#define LED		P1OUT_bit.P1OUT_0	// Output pin for LED (active high)
#define SLAVE_ADDRESS	0x49		// I2C address of this thermometer
									// Usually 0x48
#define READ_ADDRESS 	((SLAVE_ADDRESS << 1) | BIT0)
									// Address plus bit R/_W = 1
#define N_TRANSMIT	2				// Number of bytes in message
uint8_t TXdata[N_TRANSMIT] = {0,0};	// Data to transmit, initially 0

void main (void)
{
	uint32_t temperature;			// Converted value of temperature
									//   in units of 0.01degC
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	BCSCTL1 = CALBC1_1MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_1MHZ;			// Calibrated tap and modulation
	P2SEL = 0;						// Digital i/o rather than crystal
	P2DIR = 0;						// Inputs
	P2REN = BIT6 | BIT7;			// Pull resistors on unused pins
// USI will override these settings for SCL, SDA
	P1OUT = ~BIT0;					// LED low (off) others high
	P1DIR = BIT0;					// LED output, others input
	P1REN = ~BIT0;					// Pull unused pins up
// Configure SD16A: clock from SMCLK = 1MHz, internal reference on
	SD16CTL = SD16SSEL_1 | SD16REFON;
// Unipolar output, "single" convs, OSR = 1024, interrupts on finish
	SD16CCTL0 = SD16UNI | SD16SNGL | SD16OSR_1024 | SD16IE;
// PGA gain = 1, temperature sensor A6+/-, interrupts after 4th result
	SD16INCTL0 = SD16GAIN_1 | SD16INCH_6 | SD16INTDLY_0;
// No external inputs
	SD16AE = 0;
// Enable SDA, SCL, hold module in reset state
// Defaults: msb first, slave, enable output latch, disable data output
	USICTL0 = USIPE7 | USIPE6 | USISWRST;
// I2C not SPI, interrupts on start and counter, can't clear USIIFG
// Default: CPHA = 1 -> CKPH = 0
	USICTL1 = USII2C | USISTTIE | USIIE | USIIFG;
//	No clock generator for slave, clock idles high (CPOL = CKPL = 1)
	USICKCTL = USICKPL;
	USICTL0 &= ~USISWRST;			// Release USI from reset
	USICTL1 &= ~USIIFG;				// Clear flag, avoid unwanted intpt
	for (;;) {						// Loop in LPM4 with interrupts
		__low_power_mode_4();		//   (USI clocked from master)
// Return here after SD16A conversions
		temperature = SD16MEM0;		// Raw converted value
		temperature *= 45454;		// Scaling factor (Vref/temp coeff)
		temperature += 0x8000;		// Allow for rounding in division
		temperature /= 0x10000;		// Divide by range of SD16A
		temperature -= 27300;		// Subtract offset to give celsius
//		temperature = 0xA5AA;		// Dummy value for scope picture
		TXdata[0] = (uint8_t) temperature;		// Extract LSB and
		TXdata[1] = (uint8_t) (temperature >> 8);	//   MSB to transmit
	}
}
//----------------------------------------------------------------------
// ISR for USI: State machine to control I2C slave transmission
// Start condition clears USISCLREL: SCL is held until USIIFG is cleared
//----------------------------------------------------------------------
#pragma vector = USI_VECTOR
__interrupt void USI_ISR (void)		// Not acknowledged automatically
{
	static uint8_t TXcount;			// Counter for bytes transmitted
	static enum {					// States of state machine
		Idle = 0,
		ReceiveAddress = 2,
		CheckAddress = 4,
		SendFirstByte = 6,
		ReceiveAck = 8,
		CheckSendByte = 10,
		Finish = 12		
	} I2CState = Idle;				// State machine initially idle
	
	if (USICTL1 & USISTTIFG) {		// Start condition received
		LED = 1;					// LED on when I2C active
		I2CState = ReceiveAddress;	// Start state machine
	}
// Select state of machine, perform actions, move on to next state
	switch (__even_in_range(I2CState, Finish)) {
	case Idle:						// Should never happen!
		break;
	case ReceiveAddress:			// Prepare to receive address
		USICNT = USIFGCC | 8;		// Stop auto clr USIIFG, read 8 bits
		USICTL1 &= ~USISTTIFG;		// Clear flag to acknowledge Start
		I2CState = CheckAddress;	// Check received address next
		break;
	case CheckAddress:				// Check received address
		if (USISRL != READ_ADDRESS) {	// Was it our address?
			USICNT |= USISCLREL;	// No: Release hold on SCL
			I2CState = Idle;		// Return to idle state
			LED = 0;				// LED off when I2C inactive
		} else {					// Yes: send acknowledgment bit
			USISRL = 0;				// Clear bit to be sent (msb)
			USICTL0 |= USIOE;		// Enable output
			USICNT |= 1;			// Send 1 bit
			TXcount = N_TRANSMIT - 1;	// Initialize byte counter
			I2CState = SendFirstByte;	// Send first byte next
		}
		break;
	case SendFirstByte:				// Send first byte of data
		USISRL = TXdata[TXcount];	// Load most significant byte
		USICNT |= 8;				// Send 8 bits
		I2CState = ReceiveAck;		// Receive acknowledgment next
		break;
	case ReceiveAck:				// Receive acknowledgment
		USICTL0 &= ~USIOE;			// Disable output to receive Ack
		USICNT |= 1;				// Receive 1 bit
		if (TXcount == 0)		 {	// Has last byte been sent?
			I2CState = Finish;		// Yes: transmission finished
		} else {
			--TXcount;				// No: Update byte counter
			I2CState = CheckSendByte;	// Check acknowledgment next
		}
		break;
	case CheckSendByte:		// Check acknowledgment and send next byte
		if ((USISRL & BIT0) != 0) {	// Received Nack or Ack?
			USICNT |= USISCLREL;	// Nack: Release hold on SCL
			I2CState = Idle;		// Return to idle state
			LED = 0;				// LED off when I2C inactive
		} else {					// Ack: send next byte
			USISRL = TXdata[TXcount];	// Load next byte of data
			USICTL0 |= USIOE;		// Enable output
			USICNT |= 8;			// Send 8 bits
			I2CState = ReceiveAck;	// Receive acknowledgment next
		}
		break;
	case Finish:
		USICTL0 &= ~USIOE;			// Disable output to release SDA
		USICNT |= USISCLREL;		// Release hold on SCL
		I2CState = Idle;			// Return to idle state
		LED = 0;					// LED off when I2C inactive
		SD16CCTL0_bit.SD16SC = 1;	// Trigger new conversion for temp
// Change mode from LPM4 to LPM0 on exit to provide SMCLK for SD16
		__bic_SR_register_on_exit(LPM4_bits);
		__bis_SR_register_on_exit(LPM0_bits);
		break;
	}
	USICTL1 &= ~USIIFG;				// Clear flag, release SCL if held
}
//----------------------------------------------------------------------
// ISR for SD16A: return to main function to compute temperature
//----------------------------------------------------------------------
#pragma vector = SD16_VECTOR
__interrupt void SD16_ISR (void)	// Not acknowledged automatically
{
	SD16CCTL0_bit.SD16IFG = 0;		// Clear flag, acknowledge interrupt
	__low_power_mode_off_on_exit();	// Return to main() for calculation
}
