// usispimaster1.c - measure temperature and transmit as master over SPI
// 16-bit transfer, SPI mode 3 (CPOL = CPHA = 1), msb first
// Delay between readings received from SPI in multiples of 0.7s
// Vref for SD16A left running continuously for simplicity
// F2013 on TI Experimenter's Board:
//   SPI clock = SMCLK / 16 = 62.5kHz, so about (1/4) ms for transfer
//   SCLK on P1.5, MOSI (SDO) on P1.6, MISO (SDI) on P1.7
//   LED on P1.0 to show activity (needs JP2)
// Calibrated 1MHz DCO, no crystal, ACLK from VLO at 12kHz
// J H Davies, 2007-11-07; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes

#define LED		P1OUT_bit.P1OUT_0	// Output pin for LED (active high)

uint8_t delay = 0;					// Wait (delay+1) watchdog intervals
									//   between readings and transfers
void main (void)
{
	BCSCTL3 = LFXT1S_2;				// Select ACLK from VLO (no crystal)
	BCSCTL1 = CALBC1_1MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_1MHZ;			// Calibrated tap and modulation
// Watchdog as interval timer, clear, ACLK / 8192 = 1.5Hz (0.7s)
	WDTCTL = WDTPW | WDTTMSEL | WDTCNTCL | WDTSSEL | WDTIS0;
	IE1_bit.WDTIE = 1;				// WDT interval timer interrupts
	P2SEL = 0;						// Digital i/o rather than crystal
	P2DIR = 0;						// Inputs
	P2REN = BIT6 | BIT7;			// Pull resistors on unused pins
// USI will override these settings for SCLK, SDI and SDO
	P1OUT = 0;						// LED low (inactive)
	P1DIR = BIT0;					// LED output, others input
	P1REN = BIT1|BIT2|BIT3|BIT4;	// Pull resistors on unused pins
// Configure SD16A: clock from SMCLK, no division, internal reference on
	SD16CTL = SD16XDIV_0 | SD16DIV_0 | SD16SSEL_1 | SD16REFON;
// Unipolar output, "single" convs, OSR = 1024, interrupts on finish
	SD16CCTL0 = SD16UNI | SD16SNGL | SD16OSR_1024 | SD16IE;
// PGA gain = 1, temperature sensor A6+/-, interrupts after 4th result
	SD16INCTL0 = SD16GAIN_1 | SD16INCH_6 | SD16INTDLY_0;
// No external inputs
	SD16AE = 0;
// Enable SDI, SDO, SCLK, msb first, master, enable output, latch data
	USICTL0 = USIPE7 | USIPE6 | USIPE5 | USIMST | USIOE | USISWRST;
// Write then read (CPHA = 1 -> CKPH = 0), SPI not I2C, enable interrupt
	USICTL1 = USIIE | USIIFG;		// Can't clear USIIFG in reset mode
//	SCLK = SMCLK / 16, clock idles high (CPOL = CKPL = 1)
	USICKCTL = USIDIV_4 | USISSEL_2 | USICKPL;
	USICTL0 &= ~USISWRST;			// Release USI from reset
	USICTL1 &= ~USIIFG;				// Avoid unwanted interrupt
	for (;;) {						// Loop forever with interrupts
		__low_power_mode_3();		// LPM3 between interrupts
	}
}
//----------------------------------------------------------------------
// ISR for watchdog: activate SD16 to read temperature, change to LPM0
//----------------------------------------------------------------------
#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR (void)		// Acknowledged automatically
{
	if (delay > 0) {				// Waited for enough intervals?
		--delay;					// No: decrement count
	} else {						// Yes: start conversion and TX
		LED = 1;					// Turn LED on, show start
		SD16CCTL0_bit.SD16SC = 1;	// Trigger new conversion
// Change from LPM3 to LPM0 on exit to provide SMCLK for SD16 and USI
		__bic_SR_register_on_exit(LPM3_bits);
		__bis_SR_register_on_exit(LPM0_bits);
	}
}
//----------------------------------------------------------------------
// Wait for SD16A to complete conversion
//----------------------------------------------------------------------
// ISR for SD16A: compute temperature to 0.01C, transmit result over SPI
//----------------------------------------------------------------------
#pragma vector = SD16_VECTOR
__interrupt void SD16_ISR (void)	// Acknowledged when SD16MEM0 read
{
	uint32_t temperature;			// Converted value of temperature
									//   in units of 0.01degC
	temperature = SD16MEM0;			// Raw converted value
	temperature *= 45454;			// Scaling factor (Vref/temp coeff)
	temperature += 0x8000;			// Allow for rounding in division
	temperature /= 0x10000;			// Divide by range of SD16A
	temperature -= 27300;			// Subtract offset to give celsius
	USISR = (uint16_t) temperature;	// Copy lower word for transmission
	USICNT = USI16B | 16;			// Start SPI to transfer 16 bits
}
//----------------------------------------------------------------------
// Wait for USI to complete transmission
//----------------------------------------------------------------------
// ISR for USI: clear flag, store received delay, return to LPM3
//----------------------------------------------------------------------
#pragma vector = USI_VECTOR
__interrupt void USI_ISR (void)		// NOT acknowledged automatically
{
	USICTL1 &= ~USIIFG;				// Clear flag, acknowledge interrupt
	delay = USISRL;					// Lower byte gives next delay
	LED = 0;						// Turn LED off, show finish
// Change mode from LPM0 to LPM3 on exit: SMCLK no longer needed
	__bic_SR_register_on_exit(LPM0_bits);	// (not really necessary)
	__bis_SR_register_on_exit(LPM3_bits);
}
