// usispislave1.c - measure temperature and transmit as slave over SPI
// 16-bit transfer, SPI mode 0 (CPOL = CPHA = 0) with _SS, msb first
// New reading taken after previous value transmitted; Vref kept active
// F2013 on TI Experimenter's Board:
//   SCLK input on P1.5, MISO (SDO) on P1.6, MOSI (SDI) on P1.7
//   P1.4 is _SS (nSS) with interrupts; SDO released when _SS inactive
//   LED on P1.0 to show activity
// Calibrated 12MHz DCO, no crystal, no ACLK
// J H Davies, 2007-11-11; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430x20x3.h>				// Header file for this device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Integers of defined sizes

#define LED		P1OUT_bit.P1OUT_0	// Output pin for LED (active high)
#define nSS		BIT4				// Input pin for _SS on port 1

void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog
	BCSCTL1 = CALBC1_12MHZ;			// Calibrated range for DCO
	DCOCTL = CALDCO_12MHZ;			// Calibrated tap and modulation
	P2SEL = 0;						// Digital i/o rather than crystal
	P2DIR = 0;						// Inputs
	P2REN = BIT6 | BIT7;			// Pull resistors on unused pins
// USI will override these settings for SCLK, SDI and SDO
	P1OUT = 0;						// LED low (off)
	P1DIR = BIT0;					// LED output, others input
	P1REN = BIT1|BIT2|BIT3;			// Pull unused pins
	P1IE = nSS;						// Interrupts for _SS
	P1IES = 0;						// Rising edge first to initialize
	P1IFG = nSS;					// Simulate interrupt from nSS
// Configure SD16A: clock from SMCLK/12 = 1MHz, internal reference on
	SD16CTL = SD16XDIV_1 | SD16DIV_2 | SD16SSEL_1 | SD16REFON;
// Unipolar output, "single" convs, OSR = 1024, interrupts on finish
	SD16CCTL0 = SD16UNI | SD16SNGL | SD16OSR_1024 | SD16IE;
// PGA gain = 1, temperature sensor A6+/-, interrupts after 4th result
	SD16INCTL0 = SD16GAIN_1 | SD16INCH_6 | SD16INTDLY_0;
// No external inputs
	SD16AE = 0;
// Enable SDI, SDO, SCLK, msb first, slave, output not enabled, latch on
	USICTL0 = USIPE7 | USIPE6 | USIPE5 | USISWRST;		// Hold in reset
// Read then write (CPHA = 0 -> CKPH = 1), SPI not I2C, no interrupts
	USICTL1 = USICKPH | USIIFG;		// Can't clear USIIFG while reset
//	No clock generator for slave, clock idles low (CPOL = CKPL = 0)
	USICKCTL = 0;
	USICTL0 &= ~USISWRST;			// Release USI from reset
	for (;;) {						// Loop forever with interrupts
		__low_power_mode_4();		// LPM4 between interrupts, no clock
	}
}
//----------------------------------------------------------------------
// ISR for _SS on P1.4: enable SDO and SPI on falling edge (_SS active)
// Disable SDO and _SS, take new reading of temperature on rising edge
//----------------------------------------------------------------------
#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR (void)	// Not acknowledged automatically
{
	P1IFG &= ~nSS;					// Clear flag, acknowledge interrupt
	if ((P1IES & nSS) != 0) {		// Falling edge, _SS now active
		USICTL0 |= USIOE;			// Enable slave output
		USICNT = USI16B | 16;		// Start SPI to transfer 16 bits
		LED = 1;					// Turn LED on, show activity
	} else {						// Rising edge, _SS now inactive
		USICTL0 &= ~USIOE;			// Disable slave output
		USICNT = USI16B | 0;		// Abort any transfer in progress
		P1IE &= ~nSS;				// Disable _SS during conversion
		SD16CCTL0_bit.SD16SC = 1;	// Trigger new conversion for temp
// Change mode from LPM4 to LPM0 on exit to provide SMCLK for SD16
		__bic_SR_register_on_exit(LPM4_bits);
		__bis_SR_register_on_exit(LPM0_bits);
	}
	P1IES ^= nSS;					// Toggle edge sensitivity
}
//----------------------------------------------------------------------
// Wait for SD16A to complete conversion of temperature
//----------------------------------------------------------------------
// ISR for SD16A: compute temperature (units of 0.01C), move to USI
//----------------------------------------------------------------------
#pragma vector = SD16_VECTOR
__interrupt void SD16_ISR (void)	// Acknowledged when SD16MEM0 read
{
	uint32_t temperature;			// Converted value of temperature
									//   in units of 0.01degC
	temperature = SD16MEM0;			// Raw converted value
	temperature *= 45454;			// Scaling factor (Vref/temp coeff)
	temperature += 0x8000;			// Allow for rounding in division
	temperature /= 0x10000;			// Divide by range of SD16A
	temperature -= 27300;			// Subtract offset to give celsius
	USISR = (uint16_t) temperature;	// Copy lower word for transmission
//	USISR |= BITF;					// Diagnostic: Set first bit of TX
	P1IE |= nSS;					// Re-enable interrupts for _SS
	LED = 0;						// Turn LED off, show finish
// Change mode from LPM0 to LPM4 on exit: clocks no longer needed
	__bic_SR_register_on_exit(LPM0_bits);
	__bis_SR_register_on_exit(LPM4_bits);
}
