// debtim1.c - press button B1 to light LED1 with debounce
// Samples input at 5 ms intervals set by Timer_A, 32KHz ACLK
// Shift reg for debounce, different thresholds for press and release
// Olimex 1121STK board, LED1 active low on P2.3,
//   button B1 active low on P2.1
// J H Davies, 2007-04-10; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x11x1.h>				// Specific device
#include <intrinsics.h>				// Intrinsic functions
#include <stdint.h>					// Standard integer types

union {								// Debounced state of P2IN
	unsigned char DebP2IN;			// Complete byte
	struct {
		unsigned char DebP2IN_0 : 1;
		unsigned char DebP2IN_1 : 1;
		unsigned char DebP2IN_2 : 1;
		unsigned char DebP2IN_3 : 1;
		unsigned char DebP2IN_4 : 1;
		unsigned char DebP2IN_5 : 1;
		unsigned char DebP2IN_6 : 1;
		unsigned char DebP2IN_7 : 1;
	} DebP2IN_bit;					// Individual bits
};
#define RAWB1	P2IN_bit.P2IN_1
#define DEBB1	DebP2IN_bit.DebP2IN_1
#define LED1	P2OUT_bit.P2OUT_3

void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;		// Stop watchdog timer
	P2OUT_bit.P2OUT_3 = 1;			// Preload LED1 off (active low!)
	P2DIR_bit.P2DIR_3 = 1;			// Set pin with LED1 to output
	DebP2IN = 0xFF;					// Initial debounced state of port
	TACCR0 = 160;					// 160 counts at 32KHz = 5ms
	TACCTL0 = CCIE;					// Enable interrupts on Compare 0
	TACTL = MC_1|TASSEL_1|TACLR;	// Set up and start Timer A
// "Up to CCR0" mode, no clock division, clock from ACLK, clear timer
	for (;;) {						// Loop forever
		__low_power_mode_3();		// Enter LPM3, only ACLK active
// Return to main function when a debounced transition has occurred
		LED1 = DEBB1;				// Update LED1 from debounced button
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A chan 0; no need to acknowledge
// Device returns to LPM3 automatically after ISR unless input changes
// PRESS_THRESHOLD = 0x3F = 0b00111111, needs 2 successive 0s (enough?)
// RELEASE_THRESHOLD = 0xFC = 0b11111100, 6 successive 1s (too many?)
//----------------------------------------------------------------------
#define PRESS_THRESHOLD		0x3F
#define RELEASE_THRESHOLD	0xFC

#pragma vector = TIMERA0_VECTOR
__interrupt void TA0_ISR (void)
{
	static uint8_t P21ShiftReg = 0xFF;	// Shift reg for history of P2.1

	P21ShiftReg >>= 1;				// Update history in shift register
	if (RAWB1 == 1) {				// Insert latest input from B1
		P21ShiftReg |= BIT7;		// Set msb if input high
	}
	if (DEBB1 == 0) {
// Current debounced value low, looking for input to go high (release)
		if (P21ShiftReg >= RELEASE_THRESHOLD) {	// button released
			DEBB1 = 1;				// New debounced state high
			__low_power_mode_off_on_exit();	// Wake main routine
		}
	} else {
// Current debounced value high, looking for input to go low (press)
		if (P21ShiftReg <= PRESS_THRESHOLD) {	// button pressed
			DEBB1 = 0;				// New debounced state low
			__low_power_mode_off_on_exit();	// Wake main routine
		}
	}
}
