// sclock1.c - simple clock using SBLCD4 on TI Experimenter's Board
// Default clocks; ACLK from 32KHz crystal; minimal configuration
// Timer clock 32KHz ACLK, no division, up mode, period 0x8000 = 1s
// J H Davies, 2007-04-23; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430xG46x.h>					// Specific device
#include <intrinsics.h>					// Intrinsic functions
#include <stdint.h>						// Integers of defined sizes
//----------------------------------------------------------------------
#define LCDMEMS	11						// LCD memories used (3-13)
// Pointer to LCD memory used: allows use of array LCDMem[]
uint8_t * const LCDMem = (uint8_t *) &LCDM3;	
// LCD segment definitions (SoftBaugh SBLCDA4)
#define SEG_A   BIT0					//  AAAA
#define SEG_B   BIT1					// F    B
#define SEG_C   BIT2					// F    B
#define SEG_D   BIT3					//  GGGG
#define SEG_E   BIT6					// E    C
#define SEG_F   BIT4					// E    C
#define SEG_G   BIT5					//  DDDD
#define SEG_H   BIT7					// colon, point etc
// Patterns for hexadecimal characters
const uint8_t LCDHexChar[] = {
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,			// '0'
	SEG_B | SEG_C,											// '1'
	SEG_A | SEG_B | SEG_D | SEG_E | SEG_G,					// '2'
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_G,					// '3'
	SEG_B | SEG_C | SEG_F | SEG_G,							// '4'
	SEG_A | SEG_C | SEG_D | SEG_F | SEG_G,					// '5'
	SEG_A | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,			// '6'
	SEG_A | SEG_B | SEG_C,									// '7'
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,	// '8'
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G,			// '9'
	SEG_A | SEG_B | SEG_C | SEG_E | SEG_F | SEG_G,			// 'A'
	SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,					// 'b'
	SEG_A | SEG_D | SEG_E | SEG_F,							// 'C'
	SEG_B | SEG_C | SEG_D | SEG_E | SEG_G,					// 'd'
	SEG_A | SEG_D | SEG_E | SEG_F | SEG_G,					// 'E'
	SEG_A | SEG_E | SEG_F | SEG_G,							// 'F'
};
// Patterns for AM ('A'), PM ('P') and blank
const uint8_t LCDAMChar = SEG_A | SEG_B | SEG_C | SEG_E | SEG_F | SEG_G;
const uint8_t LCDPMChar = SEG_A | SEG_B | SEG_E | SEG_F | SEG_G;
const uint8_t LCDBlankChar = 0;
//----------------------------------------------------------------------
// Function prototypes
void InitLCD (void);
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW|WDTHOLD;				// Stop watchdog timer
	InitLCD();							// Initialize SBLCDA4
	TACCR0 = 0x8000;					// Upper limit of count for TAR
	TACCTL0 = CCIE;						// Enable Compare 0 interrupts
	TACTL = MC_1|TASSEL_1|TACLR;		// Set up and start Timer A
// "Up to CCR0" mode, no clock division, clock from ACLK, clear timer
	for (;;) {							// Loop forever
		__low_power_mode_3();			// ACLK continues to run
	}
}
//----------------------------------------------------------------------
// Initialize SBLCDA4
//----------------------------------------------------------------------
void InitLCD (void)
{
	int i;
	for(i = 0; i < LCDMEMS; ++i) {		// Clear LCD memories used
		LCDMem[i] = 0;
	}
	P5SEL = BIT4|BIT3|BIT2;				// Select COM[3:1] function
	LCDAPCTL0 = LCDS4|LCDS8|LCDS12|LCDS16|LCDS20|LCDS24;	
								// Enable LCD segs 4-27 (4-25 used)
	LCDAVCTL0 = 0;				// No charge pump, everything internal
	LCDACTL = LCDFREQ_128 | LCD4MUX | LCDSON | LCDON;	
								// ACLK/128, 4mux, segments on, LCD_A on
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0
// Time stored as seconds, minutes and hours in BCD format
// Need "unsigned short" to suit intrinsic function __bcd_add_short()
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)
{
	static unsigned short seconds = 0x59;	// initial time in BCD
	static unsigned short minutes = 0x59;
	static unsigned short hours = 0x23;
// Update time
	seconds = __bcd_add_short(seconds, 0x01);	// ++seconds
	if (seconds >= 0x60) {
		seconds = 0;						// start new minute
		minutes = __bcd_add_short(minutes, 0x01);
		if (minutes >= 0x60) {
			minutes = 0;					// start new hour
			hours = __bcd_add_short(hours, 0x01);
			if (hours >= 0x24) {
				hours = 0;					// start new day
			}
		}
	}
// Update all digits of display; SEG_H is colon separator
	LCDMem[1] = LCDHexChar[seconds & 0x0F];
	LCDMem[2] = LCDHexChar[(seconds >> 4) & 0x0F] | SEG_H;
	LCDMem[3] = LCDHexChar[minutes & 0x0F];
	LCDMem[4] = LCDHexChar[(minutes >> 4) & 0x0F] | SEG_H;
	LCDMem[5] = LCDHexChar[hours & 0x0F];
	LCDMem[6] = LCDHexChar[(hours >> 4) & 0x0F];
}
