// testlcd1.c - test SBLCD4 display on TI Experimenter's Board
// PCB tracks S0-S21 connected to MSP430 pins S4-S25 - beware!
// Default clocks; ACLK from 32KHz crystal; minimal configuration
// Turns on all LCD segments from least significant bit and segment,
//   then turns them off in the same sequence - a Johnson counter
// Flashes LEDs 1 and 2 on P2.2 and P2.1 to confirm operation of timer
// Timer clock 32KHz ACLK, no division, up mode, period 0x1000 = 1/8 s
// J H Davies, 2006-11-22; IAR Kickstart version 3.41A
//----------------------------------------------------------------------
#include <io430xG46x.h>					// Specific device
#include <intrinsics.h>					// Intrinsic functions
#include <stdint.h>						// Integers of defined sizes
//----------------------------------------------------------------------
#define LCDMEMS	11						// LCD memories used (3-13)
// Pointer to LCD memory used: allows use of array LCDMem[]
uint8_t * const LCDMem = (uint8_t *) &LCDM3;	
//----------------------------------------------------------------------
// Function prototypes
void InitLCD (void);
//----------------------------------------------------------------------
void main (void)
{
	int i, j;							// Loop counters
	enum {up, down} direction;			// Direction of Johnson counter

	WDTCTL = WDTPW|WDTHOLD;				// Stop watchdog timer
	P2OUT = BIT2;					// LED1 (P2.2) on, LED2 (P2.1) off
	P2DIR = BIT1|BIT2;					// Set pins with LEDs to output
	InitLCD();							// Initialize SBLCDA4
	TACCR0 = 0x1000;					// Upper limit of count for TAR
	TACCTL0 = CCIE;						// Enable Compare 0 interrupts
	TACTL = MC_1|TASSEL_1|TACLR;		// Set up and start Timer A
// "Up to CCR0" mode, no clock division, clock from ACLK, clear timer
	direction = up;						// Start Johnson counter up
	for (;;) {							// Loop forever
		for (i = 0; i < LCDMEMS; ++i) {	// Count through LCD memories
			for (j = 0; j < 8; ++j) {	// Count through bits
				__low_power_mode_3();	// LPM3 provides delay
				if (direction == up) {	// Shift left, insert 1 on right
					LCDMem[i] = (LCDMem[i] << 1) | BIT0;	
				} else {				// Shift left, insert 0 (default)
					LCDMem[i] = (LCDMem[i] << 1);
				}
			}
			__low_power_mode_3();	// extra delays after each character
			__low_power_mode_3();
		}
		if (direction == up) {		// Reverse direction after all chars
			direction = down;
		} else {
			direction = up;
		}
	}
}
//----------------------------------------------------------------------
// Initialize SBLCDA4
//----------------------------------------------------------------------
void InitLCD (void)
{
	int i;
	for(i = 0; i < LCDMEMS; ++i) {		// Clear LCD memory used
		LCDMem[i] = 0;
	}
	P5SEL = BIT4 | BIT3 | BIT2;			// Select COM[3:1] function
	LCDAPCTL0 = LCDS4|LCDS8|LCDS12|LCDS16|LCDS20|LCDS24;	
								// Enable LCD segs 4-27 (4-25 used)
	LCDAVCTL0 = 0;				// No charge pump, everything internal
	LCDACTL = LCDFREQ_128 | LCD4MUX | LCDSON | LCDON;	
}								// ACLK/128, 4mux, segments on, LCD_A on
//----------------------------------------------------------------------
// Interrupt service routine for Timer A channel 0
// Processor returns to "calling routine" in active mode
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)
{
	P2OUT ^= BIT1|BIT2;					// Toggle LEDs
	__low_power_mode_off_on_exit();		// Restore Active Mode on return
}
