// clkcmp1.c - Show ratio of SMCLK : ACLK on display using Timer_A
// Compared every 0.5s, triggered by Basic Timer; flashes LED as signal
// TI Experimenter's Board with F4619, fairly complete initialization
// J H Davies, 2007-06-03; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430xG46x.h>					// Specific device
#include <intrinsics.h>					// Intrinsic functions
#include <stdint.h>						// Integers of defined sizes
#include "LCDutils.h"					// SBLCDA4 utility functions
//----------------------------------------------------------------------
void PortsInit (void);
void TimersInit (void);
//----------------------------------------------------------------------
void main (void)
{
	volatile uint16_t i;				// Counter for FLL+ delay

	WDTCTL = WDTPW | WDTHOLD;			// Stop watchdog timer
// Clock default: MCLK = 32 x ACLK = 1MHz, FLL+ operating
	FLL_CTL0 = XCAP14PF;				// Configure load caps (14pF)
	do {
		IFG1_bit.OFIFG = 0;				// Try to clear OSCFault flag
		for (i = 50000; i > 0; --i) {	// Delay for flag to re-set
		}								//   if not yet stable
	} while (IFG1_bit.OFIFG != 0);		// OSCFault flag still set?
	PortsInit();						// Initialize ports
	LCDInit();							// Initialize SBLCDA4
	TimersInit();						// Initialize Timer_A and BT
	for (;;) {							// Loop forever with interrupts
		__low_power_mode_0();			// SMCLK and DCO continue to run
	}									//   and FLL+ remains active
}
//----------------------------------------------------------------------
// Initialize Timer_A for capture from ACLK, BT for 0.5s interrupts
//----------------------------------------------------------------------
void TimersInit (void)
{
// Initialize basic timer: ACLK, cascaded, 0.5s interrupts, stopped
	BTCTL = BTHOLD | BT_ADLY_500;
	BTCNT1 = BTCNT2 = 0;				// Clear counters
	IE2_bit.BTIE = 1;					// Enable basic timer interrupts
	BTCTL &= ~BTHOLD;					// Start basic timer running
// Capture on CCI2B, rising edge, sync; do not yet enable interrupts
	TACCTL2 = CCIS_1 | CM_1 | SCS | CAP;
// Start timer: SMCLK, no prescale, continuous mode, no ints, clear
	TACTL = TASSEL_2 | ID_0 | MC_2 | TACLR;
}
//----------------------------------------------------------------------
// Interrupt service routine for basic timer: start captures on CCI2B
// Flag cleared automatically
//----------------------------------------------------------------------
#pragma vector = BASICTIMER_VECTOR
__interrupt void BASICTIMER_ISR (void)
{
	TACCTL2_bit.COV = 0;			// Clear previous overruns
	TACCTL2_bit.CCIFG = 0;			// Clear pending interrupt
	TACCTL2_bit.CCIE = 1;			// Enable interrupts on capture
}
//----------------------------------------------------------------------
// Interrupt service routine for Timer_A, called on capture
// Read TAIV to clear flag and "service" erroneous interrupts
//----------------------------------------------------------------------
#define NCAPTURES	32				// Number of captures to accumulate
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// ISR for CCIFGn and TAIFG
{
	static uint8_t Captures = 0;	// Number of captures accumulated
	static uint16_t StartTime;		// Time at start of sampling
	
	if (TAIV == TAIV_CCIFG2) {		// CCIFG2 vector (4)
		switch (Captures) {
		case 0:						// Starting new sequence of captures
			StartTime = TACCR2;		// Starting time
			Captures = NCAPTURES;	// Initialize down counter
			P5OUT_bit.P5OUT_1 = 1;	// Light LED to mark start captures
			break;
		case 1:						// Final capture of sequence
			DisplayUint (TACCR2 - StartTime);	// Display result
			Captures = 0;			// Finished
			TACCTL2_bit.CCIE = 0;	// Disable further interrupts
			P5OUT_bit.P5OUT_1 = 0;	// Turn off LED to mark end captures
			break;
		default:					// Sequence of captures continues
			--Captures;
			break;
		}
	}
}
//----------------------------------------------------------------------
// Initialize ports for board: outputs driven low by default
// Many of these will be overwritten by LCD initialization later
//----------------------------------------------------------------------
void PortsInit (void)
{
// Port 1: 0 = SW1, 1 = SW2, rest used for Chipcon (not placed)
	P1OUT = 0;
	P1DIR = 0xFF & ~(BIT0|BIT1);		// P1.0,1 input, others output
// Port 2: 1 = LED2, 2 = LED1, 3 = mic power, 4 = UCA0TXD, 5 = UCA0RXD
	P2OUT = 0;
	P2DIR = 0xFF & ~BIT5;				// P2.5 must be input
// Port 3: 0-3 = comms with F2013, 5 = buzzer (active low)
//	P3OUT = BIT1 | BIT2;				// F2013 might tie these high
//	P3DIR = 0xFF & ~BIT5;				// P3.5 must be high
	P3OUT = BIT1 | BIT2 | BIT5;			// F2013 might tie 1,2 high;
	P3DIR = 0xFF;						//   P3.5 high for buzzer off
// Port 4: 2-5 used for Chipcon (not placed)
	P4OUT = 0;
	P4DIR = 0xFF;
// Port 5: 1 = LED4, 2-4 = COM1-3 for LCD
	P5OUT = 0;
	P5DIR = 0xFF;
// Port 6: All except 6 used for opamp external components
	P6OUT = 0;
	P6DIR = 0xFF;
// Port 7: 4-7 = LCD segments
	P7OUT = 0;
	P7DIR = 0xFF;
// Port 8: LCD segments
	P8OUT = 0;
	P8DIR = 0xFF;
// Port 9: LCD segments
	P9OUT = 0;
	P9DIR = 0xFF;
// Port 10: 0-5 = LCD segments
	P10OUT = 0;
	P10DIR = 0xFF;
}
//----------------------------------------------------------------------
// Modifications to reconfigure clock for higher frequencies
//----------------------------------------------------------------------
// To reconfigure clock for 2.1 MHz
//	FLL_CTL0 |= XCAP14PF;				// Configure load caps (14pF)
//	SCFQCTL = SCFQ_2M;					// DCO = 64 x ACLK = 2.1 MHz
// To reconfigure clock for 4.2 MHz
//	FLL_CTL0 = DCOPLUS | XCAP14PF;		// Config caps (14pF) and mult
//	SCFQCTL = SCFQ_2M;					// DCO = 64 x ACLK = 2.1 MHz
//	SCFI0 = FLLD_2 | FN_4;				// Double again, DCO range 4
