// contall1.c - All channels in operation using Timer_A continuous mode
// LED1 = TA1 in toggle mode, 50% duty cycle, period 0.5s
// LED2 = TA2 using set and reset modes, 10% duty, period roughly 10ms
// Piezo toggled by TA0 interrupts at close to 440Hz (standard 'A'),
//  turned on and off every 2s using TAIFG interrupts
// Olimex 1121STK, LED1 on P2.3 = TA1, LED2 on P2.4 = TA2, active low;
//   piezo between P2.0 and P2.5, Timer_A from ACLK = 32KHz
// J H Davies, 2007-06-19; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x11x1.h>			// Specific device
#include <intrinsics.h>			// Intrinsic functions
//----------------------------------------------------------------------
// Delays in cycles of timer clock = ACLK = 32KHz = 0x8000Hz
#define LED1half	0x2000		// (1/4)s half-period for toggling
#define LED2duty	30			// Duration of "on" duty cycle
#define	LED2period	300			// Total period
#define PIEZhalf	37			// Toggle piezo to give f = 440Hz nearly
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
// Channel 0 of timer, interrupts only to drive piezo
	TACCR0 = PIEZhalf;			// "Compare" value for first interrupt
	TACCTL0 = CCIE;				// Enable interrupts only (no output)
// Channel 1 of timer, drive LED1 (P2.3) in toggle mode with interrupts
	TACCR1 = LED1half;			// Compare value for first interrupt
	TACCTL1 = OUTMOD_4 | CCIE;	// Toggle mode with interrupts
// Channel 2 of timer, drive LED2 (P2.4) with set, reset and interrupts
	TACCR2 = LED2duty;			// Compare value for first interrupt
	TACCTL2 = OUTMOD_1 | CCIE;	// "Set" to turn LED off at next match
// Configure ports 1 and 2; redirect P2.3,4 to Timer_A
	P1OUT = BIT0 | BIT1;		// Output, high for Freq pin and TXD
	P1DIR = BIT0 | BIT1;
	P2SEL = BIT3 | BIT4;		// Re-route P2.3 to TA1, P2.4 to TA2
	P2OUT = BIT0 | BIT3 | BIT4;	// LEDs off (active low); prepare piezo
	P2DIR = BIT0 | BIT3 | BIT4 | BIT5;	// Piezo and LED outputs
// Start timer from ACLK, Continuous mode, clear, TAIFG interrupts
	TACTL = TASSEL_1 | ID_0 | MC_2 | TACLR | TAIE;
	for (;;) {					// Loop forever
		__low_power_mode_3();	// LPM3, all action in interrupts
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for TACCR0.CCIFG, called on compare
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)	// Flag cleared automatically
{
	P2OUT ^= (BIT0|BIT5);		// Toggle piezo sounder
	TACCR0 += PIEZhalf;			// Calculate next compare value
}
//----------------------------------------------------------------------
// Interrupt service routine for TACCRn.CCIFG (n > 0) and TAIFG
//----------------------------------------------------------------------
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// ISR for TACCRn CCIFG and TAIFG
{
	switch (__even_in_range(TAIV, 10)) {	// Acknowledges interrupt
	case 0:						// No interrupt pending
		break;					// No action
	case TAIV_CCIFG1:			// Vector 2: CCIFG1
		TACCR1 += LED1half;		// Calculate next compare value
		break;					// (LED1 toggled automatically)
	case TAIV_CCIFG2:			// Vector 4: CCIFG2
		if (TACCTL2 & OUTMOD_4) {	// Was last action Reset (on)?
			TACCR2 += LED2duty;		// Duration of On part of cycle
			TACCTL2 = OUTMOD_1 | CCIE;	// "Set" turns LED off next
		} else {				// Last action was Set (off)
			TACCR2 += (LED2period - LED2duty);	// Duration of Off part
			TACCTL2 = OUTMOD_5 | CCIE;	// "Reset" turns LED on next
		}
		break;
	case TAIV_TAIFG:			// Vector A: TAIFG
		TACCTL0_bit.CCIE ^= 1;	// Toggle CCIFG0 ints (piezo on/off)
		break;
	default:					// Should not be possible
		for (;;) {				// Disaster! Loop here for ever
		}
	}
}
