// doorbel2.c - accurate 0.5s doorbell after debounced press
// As doorbel1 but TAR read while running, not stopped, in software loop
// Channel 1 of Timer_A, Continuous mode, ACLK/8 = 4KHz
// Olimex 1121STK, LED1 on P2.3 = TA1, B1 on P2.1, both active low
// J H Davies, 2007-06-24; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x11x1.h>			// Specific device, newer format
#include <intrinsics.h>			// Intrinsic functions
#include <stdint.h>				// Integers of defined sizes
//----------------------------------------------------------------------
// Duration of delays in cycles of timer clock = ACLK/8 = 4KHz
#define DELAY10ms	40			// 10ms delay for debounce and pause
#define DELAY500ms	0x0800		// Duration of buzz and dead time
//----------------------------------------------------------------------
void main (void)
{
	uint16_t LastTAR;			// Copy of TAR to guard against races

	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
// Channel 1 of timer, drives LED1 (P2.3 = TA1), active low
	TACCTL1 = OUTMOD_0 | OUT;	// Force output initially high (LED off)
// Start timer from ACLK/8, continuous mode, no need to clear
	TACTL = TASSEL_1 | ID_3 | MC_2;
	P1OUT = BIT0 | BIT1;		// Output, high for Freq pin and TXD
	P1DIR = BIT0 | BIT1;
	P2SEL = BIT3;				// Re-route P2.3 to Timer TA1
	P2OUT = BIT3 | BIT4;		// LEDs off (active low)
	P2DIR = BIT0 | BIT3 | BIT4 | BIT5;	// Piezo and LED outputs
	P2IES = BIT1;				// Detect falling edge on P2.1 = B1

	for (;;) {					// Loop forever
		P2IFG = 0;				// Clear any pending interrupts on P2
		P2IE = BIT1;			// Enable interrupts on P2.1
		__low_power_mode_4();	// LPM4 until button pressed
		P2IE = 0;				// Disable further interrupts on P2
		do {						// Loop until two successive reads
			LastTAR = TAR;			//   of TAR agree with each other
		} while (LastTAR != TAR);	// Take care debugging this!
// Request interrupt after 10ms delay using TACCR1
		TACCR1 = LastTAR + DELAY10ms;	// "Compare" value for 10ms del
		TACCTL1_bit.CCIE = 1;	// Enable interrupts on compare
		__low_power_mode_3();	// LPM3 until timer interrupts
		if (P2IN_bit.P2IN_1 == 0) {	// Button still down? Continue if so
// Drive output low (reset) after 10ms delay using TACCR1
			TACCR1 += DELAY10ms;	// Further 10ms delay
			TACCTL1 = OUTMOD_5 | CCIE;	// "Reset" output, turn on LED
			__low_power_mode_3();	// LPM3 until timer interrupts
// Drive output high (set) after 500ms delay using TACCR1
			TACCR1 += DELAY500ms;	// Further 500ms delay
			TACCTL1 = OUTMOD_1 | CCIE;	// "Set" output, turn off LED
			__low_power_mode_3();	// LPM3 until timer interrupts
		}
		TACCTL1_bit.CCIE = 0;	// Disable further interrupts on CCIFG1
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for port 2; need to clear flag
//----------------------------------------------------------------------
#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR (void)	// Flag NOT cleared automatically!
{
	P2IFG = 0;						// Acknowledge interrupt
	__low_power_mode_off_on_exit();	// Return to main routine
}
//----------------------------------------------------------------------
// Interrupt service routine for CCIFG1 (only source active for vector)
//----------------------------------------------------------------------
#pragma vector = TIMERA1_VECTOR
__interrupt void TIMERA1_ISR (void)	// Flag NOT cleared automatically!
{
	TACCTL1_bit.CCIFG = 0;			// Acknowledge interrupt
	__low_power_mode_off_on_exit();
}
