// ledpwm1.c - Ramped PWM using Timer_A in Up mode from ACLK
// PWM at 128Hz, 128 counts per cycle from ACLK/2; ramp 1s up, 1s down
// Pulse width is incremented or decremented by 1 each cycle in ramp
// Duty cycle updated by TACCR0 CCIFG interrupt
// Olimex 1121STK, LED1 on P2.3 = TA1, active low; piezo off
// J H Davies, 2007-07-18; IAR Kickstart version 3.42A
//----------------------------------------------------------------------
#include <io430x11x1.h>			// Specific device
#include <intrinsics.h>			// Intrinsic functions
#include <stdint.h>				// Integers of defined sizes
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;	// Stop watchdog timer
// Set up Timer_A for PWM on active low LED, channel 1
	TACCTL1 = OUTMOD_0 | OUT;	// Force output initially high (LED off)
	TACCR0 = 0x007F;			// 128Hz from 16KHz timer clock
	TACCR1 = 0x0000;			// Duty cycle D = 0 initially
	TACCTL0 = CCIE;				// Enable interrupts on compare
	TACCTL1 = OUTMOD_3;			// Set/reset mode for negative PWM
// Configure ports 1 and 2; redirect P2.3 to Timer_A
	P1OUT = BIT0 | BIT1;		// Output high for Freq pin and TXD
	P1DIR = BIT0 | BIT1;
	P2SEL = BIT3;				// Route P2.3 to TA1
	P2OUT = BIT3 | BIT4;		// LEDs off (active low); piezo off
	P2DIR = BIT0 | BIT3 | BIT4 | BIT5;	// Piezo and LED outputs
// Start timer from ACLK/2, Up mode, clear, no interrupts
	TACTL = TASSEL_1 | ID_1 | MC_1 | TACLR;
	for (;;) {					// Loop forever
		__low_power_mode_3();	// Enter LPM3 between interrupts
	}
}
//----------------------------------------------------------------------
// Interrupt service routine for TACCR0.CCIFG
// Compiler warning for "if (TACCR1 > TACCR0)" can be ignored
//----------------------------------------------------------------------
#pragma vector = TIMERA0_VECTOR
__interrupt void TIMERA0_ISR (void)	// Flag cleared automatically
{
	static enum {up, down} direction = up;

	if (direction == up) {		// Increasing duty cycle?
		++TACCR1;				// Yes: increase TACCR1
		if (TACCR1 > TACCR0) {	// Hit 100% limit?
			direction = down;	// Yes: decrease duty cycle next
		}
	} else {
		--TACCR1;				// Decrease duty cycle
		if (TACCR1 == 0) {		// Hit 0 limit?
			direction = up;		// Yes: increase duty cycle next
		}
	}
}
