// setclk1.c - settable clock using SBLCD4 on TI Experimenter's Board
// Default clocks; ACLK from 32KHz crystal
// Periodic state machine triggered by 0.5s interrupt from basic timer
// J H Davies, 2008-01-01; IAR Kickstart version 4.09A
//----------------------------------------------------------------------
#include <io430xG46x.h>					// Specific device
#include <intrinsics.h>					// Intrinsic functions
#include <stdint.h>						// Integers of defined sizes
//----------------------------------------------------------------------
#define LCDMEMS	11						// LCD memories used (3-13)
// Pointer to LCD memory used: allows use of array LCDMem[]
uint8_t * const LCDMem = (uint8_t *) &LCDM3;
// LCD segment definitions (SoftBaugh SBLCDA4)
#define SEG_A   BIT0					//  AAAA
#define SEG_B   BIT1					// F    B
#define SEG_C   BIT2					// F    B
#define SEG_D   BIT3					//  GGGG
#define SEG_E   BIT6					// E    C
#define SEG_F   BIT4					// E    C
#define SEG_G   BIT5					//  DDDD
#define SEG_H   BIT7					// colon, point etc
// Patterns for hexadecimal characters
const uint8_t LCDHexChar[] = {
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F,			// '0'
	SEG_B | SEG_C,											// '1'
	SEG_A | SEG_B | SEG_D | SEG_E | SEG_G,					// '2'
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_G,					// '3'
	SEG_B | SEG_C | SEG_F | SEG_G,							// '4'
	SEG_A | SEG_C | SEG_D | SEG_F | SEG_G,					// '5'
	SEG_A | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,			// '6'
	SEG_A | SEG_B | SEG_C,									// '7'
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,	// '8'
	SEG_A | SEG_B | SEG_C | SEG_D | SEG_F | SEG_G,			// '9'
	SEG_A | SEG_B | SEG_C | SEG_E | SEG_F | SEG_G,			// 'A'
	SEG_C | SEG_D | SEG_E | SEG_F | SEG_G,					// 'b'
	SEG_A | SEG_D | SEG_E | SEG_F,							// 'C'
	SEG_B | SEG_C | SEG_D | SEG_E | SEG_G,					// 'd'
	SEG_A | SEG_D | SEG_E | SEG_F | SEG_G,					// 'E'
	SEG_A | SEG_E | SEG_F | SEG_G,							// 'F'
};
#define	LCDBlankChar 	0
#define	LED		P5OUT_bit.P5OUT_1		// Pin for LED
#define MAXHALFSECS		119				// Limits of counters for clock
#define	MAXMINS			0x59			// BCD for displayed values
#define	MAXHOURS		0x23
// Counters for clock, must be 16-bit variables for __bcd_add_short()
uint16_t halfsecs = MAXHALFSECS;		// Simple binary format
uint16_t minutes = MAXMINS;				// Count BCD for easier display
uint16_t hours = MAXHOURS;				// Initialize to maximum values
// Function prototypes
void PortsInit (void);					// Set up i/o ports
void InitLCD (void);					// Set up LCD
void ShowMinutes (void);				// Show minutes on LCD
void AdvanceHours (void);				// Add another hour
uint16_t AdvanceMinutes (void);			// Add another min, request hour
void AdvanceTime (void);				// Update time every 0.5s
void ActivateDisplay (void);			// Ensure display is visible
void ToggleDisplay (void);				// Toggle visibility of display
void ClearSeconds (void);				// Start new minute from 0 secs
//----------------------------------------------------------------------
void main (void)
{
	WDTCTL = WDTPW | WDTHOLD;			// Stop watchdog timer
	PortsInit();						// Initialize ports
	InitLCD();							// Initialize SBLCDA4
// Initialize basic timer: ACLK, cascaded, 0.5s interrupts, stopped
	BTCTL = BTHOLD | BTDIV | BT_fCLK2_DIV64;
	BTCNT1 = 0;							// Clear counters
	BTCNT2 = 0;
	IE2_bit.BTIE = 1;					// Enable basic timer interrupts
	IFG2_bit.BTIFG = 1;					// Interrupt to start display
	BTCTL &= ~BTHOLD;					// Start basic timer running
	for (;;) {							// Loop forever
		__low_power_mode_3();			// ACLK continues to run
	}
}
//----------------------------------------------------------------------
// Initialize ports for board: pins are outputs driven low by default
// Many of these will be overwritten by LCD initialization later
//----------------------------------------------------------------------
void PortsInit (void)
{
// Port 1: 0 = SW1, 1 = SW2, others used for Chipcon (not placed)
	P1OUT = 0;
	P1DIR = 0xFF & ~(BIT0|BIT1);		// P1.0,1 input, others output
	P1SEL = BIT0;						// P1.0 = S1 to Timer_A CCI0A
	P1IES = BIT1;						// Interrupts on falling edge
// Port 2: 1 = LED2, 2 = LED1, 3 = mic power, 4 = UCA0TXD, 5 = UCA0RXD
	P2OUT = 0;
	P2DIR = 0xFF & ~BIT5;				// P2.5 must be input
// Port 3: 0-3 = comms with F2013, 5 = buzzer (active low)
//	P3OUT = BIT1 | BIT2;				// F2013 might tie these high
//	P3DIR = 0xFF & ~BIT5;				// P3.5 must be high
	P3OUT = BIT1 | BIT2 | BIT5;			// F2013 might tie 1,2 high;
	P3DIR = 0xFF;						//   P3.5 high for buzzer off
// Port 4: 2-5 used for Chipcon (not placed)
	P4OUT = 0;
	P4DIR = 0xFF;
// Port 5: 1 = LED4, 2-4 = COM1-3 for LCD
	P5OUT = 0;
	P5DIR = 0xFF;
// Port 6: All except 6 used for opamp external components
	P6OUT = 0;
	P6DIR = 0xFF;
// Port 7: 4-7 = LCD segments
	P7OUT = 0;
	P7DIR = 0xFF;
// Port 8: LCD segments
	P8OUT = 0;
	P8DIR = 0xFF;
// Port 9: LCD segments
	P9OUT = 0;
	P9DIR = 0xFF;
// Port 10: 0-5 = LCD segments
	P10OUT = 0;
	P10DIR = 0xFF;
}
//----------------------------------------------------------------------
// Initialize SBLCDA4
//----------------------------------------------------------------------
void InitLCD (void)
{
	int i;
	for(i = 0; i < LCDMEMS; ++i) {		// Clear LCD memories used
		LCDMem[i] = 0;
	}
	P5SEL = BIT4|BIT3|BIT2;				// Select COM[3:1] function
	LCDAPCTL0 = LCDS4|LCDS8|LCDS12|LCDS16|LCDS20|LCDS24;	
								// Enable LCD segs 4-27 (4-25 used)
	LCDAVCTL0 = 0;				// No charge pump, everything internal
	LCDACTL = LCDFREQ_128 | LCD4MUX | LCDSON | LCDON;	
								// ACLK/128, 4mux, segments on, LCD_A on
}
//----------------------------------------------------------------------
// Lots of little functions to operate clock and keep state m/c concise
//----------------------------------------------------------------------
void EraseHours (void)					// Erase display for hours
{
	LCDMem[3] = LCDBlankChar;	
	LCDMem[4] = LCDBlankChar;
}
void EraseMinutes (void)				// Erase display for minutes
{										//   and colon
	LCDMem[1] = LCDBlankChar;
	LCDMem[2] = LCDBlankChar;	
}
void ShowHours (void)					// Show hours on LCD
{
	LCDMem[3] = LCDHexChar[hours & 0x0F];
	LCDMem[4] = LCDHexChar[(hours >> 4) & 0x0F];
}
void ShowMinutes (void)					// Show minutes on LCD
{
	LCDMem[1] = LCDHexChar[minutes & 0x0F];
	LCDMem[2] = LCDHexChar[(minutes >> 4) & 0x0F];
}
void AdvanceHours (void)				// Add another hour
{
	if (hours < MAXHOURS) {
		hours = __bcd_add_short(hours, 0x01);
	} else {
		hours = 0;
	}
	ShowHours();						// Update hours display
}
uint16_t AdvanceMinutes (void)			// Add another minute,
{										//   show if new hour needed
	uint16_t NewHour;
	
	if (minutes < MAXMINS) {
		minutes = __bcd_add_short(minutes, 0x01);
		NewHour = 0;
	} else {
		minutes = 0;
		NewHour = 1;
	}				
	ShowMinutes();						// Update minutes display
	return NewHour;
}
void AdvanceTime (void)					// Update time every 0.5s
{
	LCDMem[2] ^= SEG_H;					// Toggle colon separator
	if (halfsecs < MAXHALFSECS) {
		++halfsecs;
	} else {
		halfsecs = 0;
		if (AdvanceMinutes() != 0) {
			AdvanceHours();
		}
	}
}
void ActivateDisplay (void)				// Ensure display is visible
{
	LCDACTL_bit.LCDSON = 1;
}
void ToggleDisplay (void)				// Toggle visibility of display
{
	LCDACTL_bit.LCDSON ^= 1;
}
void ClearSeconds (void)				// Start new minute from 0 secs
{
	halfsecs = 0;
}
//----------------------------------------------------------------------
// Interrupt service routine for basic timer: trigger state machine
//----------------------------------------------------------------------
#define Mode		P1IN_bit.P1IN_1		// S2 = P1.1 for Mode button
#define	Advance		P1IN_bit.P1IN_0		// S1 = P1.0 for Advance button
#define	UP			1					// Inputs active low
#define	DOWN		0

#pragma vector = BASICTIMER_VECTOR
__interrupt void BASICTIMER_ISR (void)	// Acknowledged automatically
{
	static enum {
		Running,						// Normal operation of clock
		Confirm,						// Pause: mode butn still down?
		WaitHours,						// Wait for release of mode butn
		FlashHours,						// Flash settable hours display
		SetHours,						// Advance hours to set time
		WaitMinutes,					// Wait for release of mode butn
		FlashMinutes,					// Flash settable mins display
		SetMinutes,						// Advance minutes to set time
		WaitFinished					// Wait for release of mode butn
	} ClockState = Running;				// State variable
	
	switch (ClockState) {
	case Running:
		AdvanceTime();					// Always update time
		if (Mode == DOWN) {				// Mode button pressed?
			ClockState = Confirm;		// Yes: Prepare to set time
		}								// No: Continue running clock
		break;
	case Confirm:
		if (Mode == DOWN) {				// Mode button still pressed?
			EraseMinutes();				// Yes: Erase display for mins
			ClockState = WaitHours;		// Prepare to set hours
		} else {
			AdvanceTime();				// No: Update time as usual
			ClockState = Running;		// Return to timekeeping
		}
		break;
	case WaitHours:
		ToggleDisplay();				// Flash display (hours only)
		if (Mode == UP) {				// Mode button released?
			ClockState = FlashHours;	// Yes: Proceed to setting hours
		}								// No: Continue flashing display
		break;
	case FlashHours:
		if (Advance == DOWN) {			// Advance button pressed?
			ActivateDisplay();			// Yes: Ensure display is visibl
			ClockState = SetHours;		// Proceed to advance hours
		} else if (Mode == DOWN) {		// No: Mode button pressed?
			ShowMinutes();				// Yes: Show minutes
			EraseHours();				// Erase display for hours
			ActivateDisplay();			// Ensure display is visible		
			ClockState = WaitMinutes;	// Prepare to set minutes
		} else {						// No: Neither button pressed
			ToggleDisplay();			// Flash hours display
		}
		break;
	case SetHours:
		if (Advance == DOWN) {			// Advance button pressed?
			AdvanceHours();				// Yes: Advance hours
		} else {
			ClockState = FlashHours;	// No: Return to flash hours
		}
		break;
	case WaitMinutes:
		ToggleDisplay();				// Flash display (mins only)
		if (Mode == UP) {				// Mode button released?
			ClockState = FlashMinutes;	// Yes: Proceed to setting mins
		}								// No: Continue flashing display
		break;
	case FlashMinutes:
		if (Advance == DOWN) {			// Advance button pressed?
			ActivateDisplay();			// Yes: Ensure display is visibl
			ClockState = SetMinutes;	// Go to advance mins
		} else if (Mode == DOWN) {		// No: Mode button pressed?
			ShowHours();				// Yes: Restore hours to display
			ActivateDisplay();			// Ensure display is visible
			ClearSeconds();				// To give clean restart from 0s
			ClockState = WaitFinished;	// Prepare to finish setting
		} else {						// No: Neither button pressed
			ToggleDisplay();			// Flash minutes display
		}
		break;
	case SetMinutes:
		if (Advance == DOWN) {			// Advance button pressed?
			AdvanceMinutes();			// Yes: Advance minutes
		} else {
			ClockState = FlashMinutes;	// No: Return to flash minutes
		}
		break;
	case WaitFinished:
		if (Mode == UP) {				// Mode button released?
			AdvanceTime();				// Yes: Restart update of time
			ClockState = Running;		// Return to running state
		}								// No: Do nothing
		break;
	default:							// Should never happen!
		ClockState = Running;			// Emergency recovery
		break;
	}
}
